@@ -1,4 +1,5 @@
-/* $Id: rrd_helpers.c 1651 2008-08-10 12:41:37Z carenas $ */
+/* $Id: rrd_helpers.c 2201 2010-01-08 17:17:43Z d_pocock $ */
+#include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -176,32 +177,39 @@ write_data_to_rrd ( const char *source, const char *host, const char *metric,
                     const char *sum, const char *num, unsigned int step,
                     unsigned int process_time, ganglia_slope_t slope)
 {
-   char rrd[ PATHSIZE ];
+   char rrd[ PATHSIZE + 1 ];
    char *summary_dir = "__SummaryInfo__";
+   int i;
 
    /* Build the path to our desired RRD file. Assume the rootdir exists. */
-   strcpy(rrd, gmetad_config.rrd_rootdir);
+   strncpy(rrd, gmetad_config.rrd_rootdir, PATHSIZE);
 
    if (source) {
-      strncat(rrd, "/", PATHSIZE);
-      strncat(rrd, source, PATHSIZE);
+      strncat(rrd, "/", PATHSIZE-strlen(rrd));
+      strncat(rrd, source, PATHSIZE-strlen(rrd));
       my_mkdir( rrd );
    }
 
    if (host) {
-      strncat(rrd, "/", PATHSIZE);
-      strncat(rrd, host, PATHSIZE);
+      strncat(rrd, "/", PATHSIZE-strlen(rrd));
+      i = strlen(rrd);
+      strncat(rrd, host, PATHSIZE-strlen(rrd));
+      if(gmetad_config.case_sensitive_hostnames == 0) {
+         /* Convert the hostname to lowercase */
+         for( ; rrd[i] != 0; i++)
+            rrd[i] = tolower(rrd[i]);
+      }
       my_mkdir( rrd );
    }
    else {
-      strncat(rrd, "/", PATHSIZE);
-      strncat(rrd, summary_dir, PATHSIZE);
+      strncat(rrd, "/", PATHSIZE-strlen(rrd));
+      strncat(rrd, summary_dir, PATHSIZE-strlen(rrd));
       my_mkdir( rrd );
    }
 
-   strncat(rrd, "/", PATHSIZE);
-   strncat(rrd, metric, PATHSIZE);
-   strncat(rrd, ".rrd", PATHSIZE);
+   strncat(rrd, "/", PATHSIZE-strlen(rrd));
+   strncat(rrd, metric, PATHSIZE-strlen(rrd));
+   strncat(rrd, ".rrd", PATHSIZE-strlen(rrd));
 
    return push_data_to_rrd( rrd, sum, num, step, process_time, slope);
 }
