@@ -5,7 +5,7 @@
 #include <apr_network_io.h>
 
 apr_socket_t *
-create_udp_client(apr_pool_t *context, char *ipaddr, apr_port_t port);
+create_udp_client(apr_pool_t *context, char *ipaddr, apr_port_t port, const char *interface, char *bind_address, int bind_hostname);
 
 apr_socket_t *
 create_udp_server(apr_pool_t *context, int32_t family, apr_port_t port, char *bind);
@@ -14,7 +14,7 @@ APR_DECLARE(apr_status_t)
 apr_sockaddr_ip_buffer_get(char *addr, int len, apr_sockaddr_t *sockaddr);
 
 apr_socket_t *
-create_mcast_client(apr_pool_t *context, char *mcast_ip, apr_port_t port, int ttl, const char *interface);
+create_mcast_client(apr_pool_t *context, char *mcast_ip, apr_port_t port, int ttl, const char *interface, char *bind_address, int bind_hostname);
 
 int mcast_set_ttl(apr_socket_t *socket, int val);
 
