@@ -1,11 +1,11 @@
-/* $Id: become_a_nobody.c 394 2004-11-18 20:14:34Z massie $ */
+/* $Id: become_a_nobody.c 1171 2008-03-31 22:00:49Z bnicholes $ */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 #include <stdio.h>
 #include "become_a_nobody.h"
-#include "error.h"
+#include <gm_msg.h>
 
 void
 become_a_nobody( const char *username )
