@@ -1,7 +1,7 @@
 /**
  * @file daemon_init.c Functions for standalone daemons
  */ 
-/* $Id: daemon_init.c 599 2005-10-31 10:52:21Z knobi1 $ */
+/* $Id: daemon_init.c 1666 2008-08-13 11:01:03Z carenas $ */
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -15,8 +15,9 @@
 #include <errno.h>
 
 #include "daemon_init.h"
+#include <gm_msg.h>
 
-#define	MAXFD	64
+#define MAXFD 64
 
 /**
  * @fn void update_pidfile (const char *pname)
@@ -26,6 +27,7 @@ extern pid_t getpgid(pid_t pid);
 void
 update_pidfile (char *pidfile)
 {
+  long p;
   pid_t pid;
   mode_t prev_umask;
   FILE *file;
@@ -34,12 +36,12 @@ update_pidfile (char *pidfile)
   file = fopen (pidfile, "r");
   if (file)
     {
-      if (fscanf(file, "%d", &pid) == 1 &&
-	  (getpgid (pid) > -1))
-	{
-	  fprintf (stderr, "daemon already running: %s pid %d\n", pidfile, pid);
-	  exit (1);
-	}
+      if (fscanf(file, "%ld", &p) == 1 && (pid = p) && 
+          (getpgid (pid) > -1))
+        {
+          err_msg("daemon already running: %s pid %ld\n", pidfile, p);
+          exit (1);
+        }
       fclose (file);
     }
 
@@ -50,17 +52,17 @@ update_pidfile (char *pidfile)
   file = fopen (pidfile, "w");
   if (!file)
     {
-      fprintf (stderr, "Error writing pidfile '%s' -- %s\n",
-	       pidfile, strerror (errno));
+      err_msg("Error writing pidfile '%s' -- %s\n",
+               pidfile, strerror (errno));
       exit (1);
     }
-  fprintf (file, "%d\n", (int) getpid());
+  fprintf (file, "%ld\n", (long) getpid());
   fclose (file);
   umask (prev_umask);
 }
 
 
-extern int daemon_proc;		/* defined in error.c */
+extern int daemon_proc;		/* defined in error_msg.c */
 
 /**
  * @fn void daemon_init (const char *pname, int facility)
@@ -76,21 +78,21 @@ daemon_init (const char *pname, int facility)
    pid = fork();
 
    if (pid != 0)
-      exit (0);			/* parent terminates */
+      exit (0);         /* parent terminates */
 
    /* 41st child continues */
-   setsid ();			/* become session leader */
+   setsid ();           /* become session leader */
 
    signal (SIGHUP, SIG_IGN);
    if ((pid = fork ()) != 0)
-      exit (0);			/* 1st child terminates */
+      exit (0);         /* 1st child terminates */
 
    /* 42nd child continues */
-   daemon_proc = 1;		/* for our err_XXX() functions */
+   daemon_proc = 1;     /* for our err_XXX() functions */
 
-   chdir ("/");			/* change working directory */
+   i = chdir ("/");     /* change working directory */
 
-   umask (0);			/* clear our file mode creation mask */
+   umask (0);           /* clear our file mode creation mask */
 
    for (i = 0; i < MAXFD; i++)
       close (i);
