@@ -1,7 +1,7 @@
 /**
  * @file debug_msg.c Debug Message function
  */
-/* $Id: debug_msg.c 394 2004-11-18 20:14:34Z massie $ */
+/* $Id: debug_msg.c 837 2007-09-19 23:29:51Z bnicholes $ */
 #include <stdio.h>
 #include <stdarg.h>
 
@@ -26,3 +26,17 @@ debug_msg(const char *format, ...)
       } 
    return;
 }
+
+void
+set_debug_msg_level(int level)
+{
+    debug_level = level;
+    return;
+}
+
+int
+get_debug_msg_level()
+{
+    return debug_level;
+}
+
