@@ -21,6 +21,10 @@
 
 /* -- dotconf.c - this code is responsible for the input, parsing and dispatching of options  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -226,7 +230,7 @@ void dotconf_register_options(configfile_t *configfile, const configoption_t * o
 	{
 		if ( !(num % GROW_BY) )
 			configfile->config_options = realloc(configfile->config_options,
-											 sizeof(void *) * (num + GROW_BY));
+											 sizeof(void *) * (num + GROW_BY + 1));
 	}
 
 #undef GROW_BY
@@ -608,7 +612,7 @@ const char *dotconf_handle_command(configfile_t *configfile, char *buffer)
 	skip_whitespace(&cp1, (long)eob - (long)cp1, 0);
 
 	/* ignore comments and empty lines */
-	if (!cp1 || !*cp1 || *cp1 == '#' || *cp1 == '\n' || *cp1 == EOF)
+	if (!cp1 || !*cp1 || *cp1 == '#' || *cp1 == '\n' || *cp1 == (char)EOF)
 		return NULL;
 
 	/* skip line if it only contains whitespace */
@@ -1064,9 +1068,11 @@ int dotconf_handle_question_mark(command_t* cmd, char* path, char* pre, char* ex
 					}
 
 				}
-
+#ifdef HAVE_SNPRINTF
+				snprintf(new_path, new_path_len, "%s%s", path, dirptr->d_name);
+#else
 				sprintf(new_path,"%s%s",path,dirptr->d_name);
-
+#endif
 				if (access(new_path, R_OK))
 				{
 					dotconf_warning(cmd->configfile, DCLOG_WARNING, ERR_INCLUDE_ERROR,
@@ -1207,7 +1213,11 @@ int dotconf_handle_star(command_t* cmd, char* path, char* pre, char* ext)
 
 					strncpy(new_pre,dirptr->d_name,(sub_count+t_ext_count));
 					new_pre[sub_count+t_ext_count] = '\0';
+#ifdef HAVE_STRLCAT
+  					strlcat(new_pre,new_ext,CFG_MAX_FILENAME);
+#else
 					strcat(new_pre,new_ext);
+#endif
 
 					sprintf(new_path,"%s%s%s",path,new_pre,t_ext);
 
