@@ -1,7 +1,7 @@
 /**
  * @file file.c Some file functions
  */
-/* $Id: file.c 816 2007-07-17 00:52:35Z bernardli $ */
+/* $Id: file.c 1938 2009-01-05 11:34:25Z carenas $ */
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -12,8 +12,7 @@
 #include <fcntl.h>
 #include <ctype.h>
 
-#include "error.h"
-#include "ganglia.h"
+#include "ganglia_priv.h"
 
 
 /**
@@ -96,7 +95,7 @@ writen (int fd, const void *vptr, size_t n)
  * @param buffer A pointer to the data buffer
  * @param buflen The data buffer length
  * @return int
- * @retval 0 on success
+ * @retval number of bytes read on success
  * @retval -1 on failure
  */
 int
@@ -121,25 +120,28 @@ slurpfile ( char * filename, char *buffer, int buflen )
          close(fd);
          return SYNAPSE_FAILURE;
       }
-   close(fd);
-
+   if (read_len == buflen)
+      {
+         --read_len;
+         err_msg("slurpfile() read() buffer overflow on file %s", filename);
+      }
    buffer[read_len] = '\0';
+   close(fd);
    return read_len;
 }   
 
 
 char * 
-skip_whitespace ( const char *p)
+skip_whitespace (const char *p)
 {
-    while (isspace(*p)) p++;
+    while (isspace((unsigned char)*p)) p++;
     return (char *)p;
 }
  
 char * 
-skip_token ( const char *p)
+skip_token (const char *p)
 {
-    while (isspace(*p)) p++;
-    while (*p && !isspace(*p)) p++;
+    while (isspace((unsigned char)*p)) p++;
+    while (*p && !isspace((unsigned char)*p)) p++;
     return (char *)p;
 }         
-
