@@ -1,21 +1,18 @@
 /**
  * @file gexec_funcs.c Functions to support gexec, gstat et al
  */
-/* $Id: ganglia.c 578 2005-08-25 20:06:10Z knobi1 $ */
+/* $Id: ganglia.c 1398 2008-06-10 17:07:25Z bernardli $ */
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 #include <time.h>
+#include <expat.h>
 
-
+#include "ganglia_gexec.h"
+#include "ganglia_priv.h"
 #include "net.h"
 #include "llist.h"
 #include "hash.h"
-#include "ganglia.h"
-#include "ganglia_priv.h"
-#include "debug_msg.h"
-#include "error.h"
-#include "expat.h"
 
 int gexec_errno = 0;
 
@@ -139,7 +136,7 @@ start (void *data, const char *el, const char **attr)
                   cluster->host->gexec_on = 1;
             }
       }
-}				
+}
 
 static void
 end (void *data, const char *el)
@@ -191,7 +188,7 @@ end (void *data, const char *el)
          cluster->num_dead_hosts++;
          llist_add((llist_entry **) &(cluster->dead_hosts), e);
       }
-}				
+}
 
 int
 gexec_cluster_free ( gexec_cluster_t *cluster )
@@ -317,8 +314,8 @@ gexec_cluster (gexec_cluster_t *cluster, char *ip, unsigned short port)
            {
               gexec_errno = 7;
               err_msg ("gexec_cluster() XML_ParseBuffer() error at line %d:\n%s\n",
-		                   XML_GetCurrentLineNumber (xml_parser),
-		                   XML_ErrorString (XML_GetErrorCode (xml_parser)));
+              XML_GetCurrentLineNumber (xml_parser),
+              XML_ErrorString (XML_GetErrorCode (xml_parser)));
               goto error;
            }
 
