@@ -1,5 +1,5 @@
 /*
- * $Id: tcp.c 394 2004-11-18 20:14:34Z massie $
+ * $Id: tcp.c 1237 2008-04-13 09:18:00Z carenas $
  * This code was originally written by the authors below but 
  * I rewrote it and took out all the glib dependencies. 
  *
@@ -24,7 +24,7 @@
  */
 #include <unistd.h>
 
-#include "error.h"
+#include <gm_msg.h>
 #include "net.h"
 
 g_tcp_socket*
@@ -200,7 +200,7 @@ g_tcp_socket_server_new_interface (const g_inet_addr* iface)
   return s;
   
  error:
-  if (s)    		free(s);
+  if (s) free(s);
   return NULL;
 }
 
