@@ -1,27 +1,42 @@
+/*
+ * This file contains all the metrics gathering code from Windows using cygwin
+ * or native Windows calls when possible.
+ * 
+ * Tested with cygwin 1.5.23-2 in Windows XP Home SP2 (i386)
+ */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
 #include <ctype.h>
+#define _WIN32_WINNT 0x0500
 #include <windows.h>
 #include <iphlpapi.h>
+#include <sys/types.h>
+#include <sys/utsname.h>
 #include <sys/timeb.h>
+#include <mntent.h>
+#include <sys/vfs.h>
+#include <psapi.h>
 
 /* From old ganglia 2.5.x... */
 #include "file.h"
 #include "libmetrics.h"
 /* End old ganglia 2.5.x headers */
-
-#define OSNAME "Cygwin"
-#define OSNAME_LEN strlen(OSNAME)
+#undef min
+#undef max
+#include "interface.h"
 
 /* Never changes */
 #ifndef BUFFSIZE
 #define BUFFSIZE 8192
 #endif
 char proc_cpuinfo[BUFFSIZE];
-char proc_sys_kernel_osrelease[BUFFSIZE];
+
+char sys_osname[MAX_G_STRING_SIZE];
+char sys_osrelease[MAX_G_STRING_SIZE];
 
 typedef struct {
   uint32_t last_read;
@@ -31,8 +46,6 @@ typedef struct {
 } timely_file;
 
 timely_file proc_stat    = { 0, 1, "/proc/stat" };
-timely_file proc_loadavg = { 0, 5, "/proc/loadavg" };
-timely_file proc_meminfo = { 0, 5, "/proc/meminfo" };
 
 static time_t
 get_netbw(double *in_bytes, double *out_bytes,
@@ -48,13 +61,13 @@ get_netbw(double *in_bytes, double *out_bytes,
   dwSize = sizeof(MIB_IFTABLE);
   
   iftable = (PMIB_IFTABLE) malloc (dwSize);
-  while (ret = GetIfTable(iftable, &dwSize, 1) == ERROR_INSUFFICIENT_BUFFER) {
+  while ((ret = GetIfTable(iftable, &dwSize, 1)) == ERROR_INSUFFICIENT_BUFFER) {
      iftable = (PMIB_IFTABLE) realloc (iftable, dwSize);
   }
  
   if (ret == NO_ERROR) { 
 
-    _ftime ( &timebuffer );
+    ftime ( &timebuffer );
 
     /* scan the interface table */
     for (dwInterface = 0; dwInterface < (iftable -> dwNumEntries); dwInterface++) {
@@ -91,7 +104,7 @@ char *update_file(timely_file *tf)
     rval = slurpfile(tf->name, tf->buffer, BUFFSIZE);
     if(rval == SYNAPSE_FAILURE) {
       err_msg("update_file() got an error from slurpfile() reading %s",
-	      tf->name);
+              tf->name);
     }
     else tf->last_read = now;
   }
@@ -122,8 +135,7 @@ num_cpustates_func ( void )
    p = skip_whitespace(p);
 /*
 ** Loop over file until next "cpu" token is found.
-** i=4 : Linux 2.4.x
-** i=7 : Linux 2.6.x
+** i=4 : Cygwin
 */
    while (strncmp(p,"cpu",3)) {
      p = skip_token(p);
@@ -140,18 +152,26 @@ num_cpustates_func ( void )
 g_val_t
 metric_init(void)
 {
+   struct utsname u;
    g_val_t rval;
 
    num_cpustates = num_cpustates_func();
 
    rval.int32 = slurpfile("/proc/cpuinfo", proc_cpuinfo, BUFFSIZE);
-   if ( rval.int32 == SYNAPSE_FAILURE )
-      {
+   if ( rval.int32 == SYNAPSE_FAILURE ) {
          err_msg("metric_init() got an error from slurpfile() /proc/cpuinfo");
          return rval;
-      }  
+   }  
 
-   strcpy( proc_sys_kernel_osrelease, "cygwin" );
+   if (uname(&u) == -1) {
+      strncpy(sys_osname, "unknown", MAX_G_STRING_SIZE);
+      strncpy(sys_osrelease, "unknown", MAX_G_STRING_SIZE);
+   } else {
+      strncpy(sys_osname, u.sysname, MAX_G_STRING_SIZE);
+      sys_osname[MAX_G_STRING_SIZE - 1] = '\0';
+      strncpy(sys_osrelease, u.release, MAX_G_STRING_SIZE);
+      sys_osrelease[MAX_G_STRING_SIZE - 1] = '\0';
+   }
 
    rval.int32 = SYNAPSE_SUCCESS;
    return rval;
@@ -260,20 +280,17 @@ bytes_in_func ( void )
 g_val_t
 cpu_num_func ( void )
 {
-#if 0
-   static int cpu_num = 0;
-#endif
+   static DWORD cpu_num = 0;
+   SYSTEM_INFO siSysInfo;
    g_val_t val;
 
-#if 0
    /* Only need to do this once */
-   if (! cpu_num) {
-      /* We'll use _SC_NPROCESSORS_ONLN to get operating cpus */
-      cpu_num = get_nprocs();
+   if (!cpu_num) {
+      GetSystemInfo(&siSysInfo);
+      cpu_num = siSysInfo.dwNumberOfProcessors;
    }
-#endif
-   val.uint16 = 1;/**FIXME: assuming only one processor right now */
 
+   val.uint16 = cpu_num;
    return val;
 }
 
@@ -337,32 +354,39 @@ cpu_speed_func ( void )
 g_val_t
 mem_total_func ( void )
 {
-   char *p;
+   MEMORYSTATUSEX stat;
+   DWORDLONG size;
    g_val_t val;
 
-   p = strstr( update_file(&proc_meminfo), "MemTotal:");
-   if(p) {
-     p = skip_token(p);
-     val.uint32 = strtol( p, (char **)NULL, 10 );
+   stat.dwLength = sizeof(stat);
+
+   if ( GlobalMemoryStatusEx (&stat)) {
+      size = stat.ullTotalPhys;
+      /* get the value in kB */ 
+      val.f =  size / 1024;
    } else {
-     val.uint32 = 0;
+      val.f = 0;
    }
 
    return val;
 }
 
+/* FIXME?: should be using PERFORMANCE_INFORMATION.CommitLimit */
 g_val_t
 swap_total_func ( void )
 {
-   char *p;
+   MEMORYSTATUSEX stat;
+   DWORDLONG size;
    g_val_t val;
- 
-   p = strstr( update_file(&proc_meminfo), "SwapTotal:" );
-   if(p) {
-     p = skip_token(p);
-     val.uint32 = strtol( p, (char **)NULL, 10 );  
+
+   stat.dwLength = sizeof(stat);
+
+   if ( GlobalMemoryStatusEx (&stat)) {
+      size = stat.ullTotalPageFile;
+      /* get the value in kB */ 
+      val.f =  size / 1024;
    } else {
-     val.uint32 = 0;
+      val.f = 0;
    }
 
    return val;
@@ -379,7 +403,7 @@ boottime_func ( void )
    p = strstr ( p, "btime" );
    if(p) { 
      p = skip_token ( p );
-     val.uint32 = strtod ( p, (char **)NULL );
+     val.uint32 = atoi ( p );
    } else {
      val.uint32 = 0;
    }
@@ -399,41 +423,37 @@ sys_clock_func ( void )
 g_val_t
 machine_type_func ( void )
 {
+   SYSTEM_INFO siSysInfo;
    g_val_t val;
  
-#ifdef __i386__
-   snprintf(val.str, MAX_G_STRING_SIZE, "x86");
-#endif
-#ifdef __x86_64__
-   snprintf(val.str, MAX_G_STRING_SIZE, "x86_64");
-#endif
-#ifdef __ia64__
-   snprintf(val.str, MAX_G_STRING_SIZE, "ia64");
-#endif
-#ifdef __sparc__
-   snprintf(val.str, MAX_G_STRING_SIZE, "sparc");
-#endif
-#ifdef __alpha__
-   snprintf(val.str, MAX_G_STRING_SIZE, "alpha");
-#endif
-#ifdef __powerpc__
-   snprintf(val.str, MAX_G_STRING_SIZE, "powerpc");
-#endif
-#ifdef __m68k__
-   snprintf(val.str, MAX_G_STRING_SIZE, "m68k");
-#endif
-#ifdef __mips__
-   snprintf(val.str, MAX_G_STRING_SIZE, "mips");
-#endif
-#ifdef __arm__
-   snprintf(val.str, MAX_G_STRING_SIZE, "arm");
-#endif
-#ifdef __hppa__
-   snprintf(val.str, MAX_G_STRING_SIZE, "hppa");
-#endif
-#ifdef __s390__
-   snprintf(val.str, MAX_G_STRING_SIZE, "s390");
-#endif
+   GetSystemInfo(&siSysInfo);
+
+   switch (siSysInfo.wProcessorArchitecture) {
+      case PROCESSOR_ARCHITECTURE_AMD64:
+         snprintf(val.str, MAX_G_STRING_SIZE, "x86_64");
+         break;
+      case PROCESSOR_ARCHITECTURE_IA64:
+         snprintf(val.str, MAX_G_STRING_SIZE, "ia64");
+         break;
+      case PROCESSOR_ARCHITECTURE_INTEL:
+         snprintf(val.str, MAX_G_STRING_SIZE, "x86");
+         break;
+      case PROCESSOR_ARCHITECTURE_ALPHA:
+      case PROCESSOR_ARCHITECTURE_ALPHA64:
+         snprintf(val.str, MAX_G_STRING_SIZE, "alpha");
+         break;
+      case PROCESSOR_ARCHITECTURE_PPC:
+         snprintf(val.str, MAX_G_STRING_SIZE, "powerpc");
+         break;
+      case PROCESSOR_ARCHITECTURE_MIPS:
+         snprintf(val.str, MAX_G_STRING_SIZE, "mips");
+         break;
+      case PROCESSOR_ARCHITECTURE_ARM:
+         snprintf(val.str, MAX_G_STRING_SIZE, "arm");
+         break;
+      default:
+         snprintf(val.str, MAX_G_STRING_SIZE, "unknown");
+   }
 
    return val;
 }
@@ -442,7 +462,9 @@ g_val_t
 os_name_func ( void )
 {
    g_val_t val;
-   snprintf(val.str, MAX_G_STRING_SIZE, "Cygwin");
+
+   snprintf(val.str, MAX_G_STRING_SIZE, "%s", sys_osname);
+
    return val;
 }
 
@@ -450,11 +472,12 @@ g_val_t
 os_release_func ( void )
 {
    g_val_t val;
-   snprintf(val.str, MAX_G_STRING_SIZE, "%s", proc_sys_kernel_osrelease);
+
+   snprintf(val.str, MAX_G_STRING_SIZE, "%s", sys_osrelease);
+
    return val;
 }
 
-
 /*
  * A helper function to return the total number of cpu jiffies
  */
@@ -462,8 +485,7 @@ unsigned long
 total_jiffies_func ( void )
 {
    char *p;
-   unsigned long user_jiffies, nice_jiffies, system_jiffies, idle_jiffies,
-                 wio_jiffies, irq_jiffies, sirq_jiffies;
+   unsigned long user_jiffies, nice_jiffies, system_jiffies, idle_jiffies;
 
    p = update_file(&proc_stat);
    p = skip_token(p);
@@ -476,21 +498,9 @@ total_jiffies_func ( void )
    p = skip_whitespace(p);
    idle_jiffies = strtod( p , &p );
 
-   if(num_cpustates == NUM_CPUSTATES_24X)
-     return user_jiffies + nice_jiffies + system_jiffies + idle_jiffies;
-
-   p = skip_whitespace(p);
-   wio_jiffies = strtod( p , &p );
-   p = skip_whitespace(p);
-   irq_jiffies = strtod( p , &p );
-   p = skip_whitespace(p);
-   sirq_jiffies = strtod( p , &p );
-  
-   return user_jiffies + nice_jiffies + system_jiffies + idle_jiffies +
-          wio_jiffies + irq_jiffies + sirq_jiffies; 
+   return (user_jiffies + nice_jiffies + system_jiffies + idle_jiffies);
 }   
 
-
 g_val_t
 cpu_user_func ( void )
 {
@@ -630,223 +640,134 @@ cpu_idle_func ( void )
    return val;
 }
 
+/* FIXME? */
 g_val_t 
 cpu_aidle_func ( void )
 {
-   char *p;
    g_val_t val;
-   double idle_jiffies, total_jiffies;
-   
-   p = update_file(&proc_stat);
 
-   p = skip_token(p);
-   p = skip_token(p);
-   p = skip_token(p);
-   p = skip_token(p);
-   idle_jiffies  = strtod( p , (char **)NULL );
-   total_jiffies = total_jiffies_func();
-   
-   val.f = (idle_jiffies/total_jiffies)*100;
+   val.f = 0.0;
+
    return val;
 }
 
+/* FIXME? */
 g_val_t 
 cpu_wio_func ( void )
 {
-   char *p;
-   static g_val_t val;
-   static int stamp;
-   static double last_wio_jiffies,  wio_jiffies,
-                 last_total_jiffies, total_jiffies, diff;
+   g_val_t val;
  
-   if (num_cpustates == NUM_CPUSTATES_24X) {
-     val.f = 0.;
-     return val;
-     }
+   val.f = 0.0;
 
-   p = update_file(&proc_stat);
-   if(proc_stat.last_read != stamp) {
-     stamp = proc_stat.last_read;
-     
-     p = skip_token(p);
-     p = skip_token(p);
-     p = skip_token(p);
-     p = skip_token(p);
-     p = skip_token(p);
-     wio_jiffies  = strtod( p , (char **)NULL );
-     total_jiffies = total_jiffies_func();
-     
-     diff = wio_jiffies - last_wio_jiffies;
-     
-     if ( diff ) 
-       val.f = (diff/(total_jiffies - last_total_jiffies))*100;
-     else
-       val.f = 0.0;
-     
-     last_wio_jiffies  = wio_jiffies;
-     last_total_jiffies = total_jiffies;
-     
-   }
-   
    return val;
 }
 
+/* FIXME? */
 g_val_t 
 cpu_intr_func ( void )
 {
-   char *p;
-   static g_val_t val;
-   static int stamp;
-   static double last_intr_jiffies,  intr_jiffies,
-                 last_total_jiffies, total_jiffies, diff;
- 
-   if (num_cpustates == NUM_CPUSTATES_24X) {
-     val.f = 0.;
-     return val;
-     }
+   g_val_t val;
+
+   val.f = 0.0;
 
-   p = update_file(&proc_stat);
-   if(proc_stat.last_read != stamp) {
-     stamp = proc_stat.last_read;
-     
-     p = skip_token(p);
-     p = skip_token(p);
-     p = skip_token(p);
-     p = skip_token(p);
-     p = skip_token(p);
-     p = skip_token(p);
-     intr_jiffies  = strtod( p , (char **)NULL );
-     total_jiffies = total_jiffies_func();
-     
-     diff = intr_jiffies - last_intr_jiffies;
-     
-     if ( diff ) 
-       val.f = (diff/(total_jiffies - last_total_jiffies))*100;
-     else
-       val.f = 0.0;
-     
-     last_intr_jiffies  = intr_jiffies;
-     last_total_jiffies = total_jiffies;
-     
-   }
-   
    return val;
 }
 
+/* FIXME? */
 g_val_t 
 cpu_sintr_func ( void )
 {
-   char *p;
-   static g_val_t val;
-   static int stamp;
-   static double last_sintr_jiffies,  sintr_jiffies,
-                 last_total_jiffies, total_jiffies, diff;
- 
-   if (num_cpustates == NUM_CPUSTATES_24X) {
-     val.f = 0.;
-     return val;
-     }
+   g_val_t val;
+
+   val.f = 0.0;
 
-   p = update_file(&proc_stat);
-   if(proc_stat.last_read != stamp) {
-     stamp = proc_stat.last_read;
-     
-     p = skip_token(p);
-     p = skip_token(p);
-     p = skip_token(p);
-     p = skip_token(p);
-     p = skip_token(p);
-     p = skip_token(p);
-     p = skip_token(p);
-     sintr_jiffies  = strtod( p , (char **)NULL );
-     total_jiffies = total_jiffies_func();
-     
-     diff = sintr_jiffies - last_sintr_jiffies;
-     
-     if ( diff ) 
-       val.f = (diff/(total_jiffies - last_total_jiffies))*100;
-     else
-       val.f = 0.0;
-     
-     last_sintr_jiffies  = sintr_jiffies;
-     last_total_jiffies = total_jiffies;
-     
-   }
-   
    return val;
 }
 
+/* FIXME? */
 g_val_t
 load_one_func ( void )
 {
    g_val_t val;
 
-   val.f = strtod( update_file(&proc_loadavg), (char **)NULL);
+   val.f = 0.0;
 
    return val;
 }
 
+/* FIXME? */
 g_val_t
 load_five_func ( void )
 {
-   char *p;
    g_val_t val;
 
-   p = update_file(&proc_loadavg);
-   p = skip_token(p);
-   val.f = strtod( p, (char **)NULL);
+   val.f = 0.0;
  
    return val;
 }
 
+/* FIXME? */
 g_val_t
 load_fifteen_func ( void )
 {
-   char *p;
    g_val_t val;
 
-   p = update_file(&proc_loadavg);
+   val.f = 0.0;
  
-   p = skip_token(p);
-   p = skip_token(p);
-   val.f = strtod( p, (char **)NULL);
-
    return val;
 }
 
+#define MAXPROCESSES 1024
+
+/* FIXME */
 g_val_t
 proc_run_func( void )
 {
-   char *p;
+   DWORD aProcesses[MAXPROCESSES], cbNeeded, cProcesses;
+   unsigned int i, running = 0;
+   HANDLE hProcess;
+   BOOL bResult;
    g_val_t val;
 
-   p = update_file(&proc_loadavg);
-   p = skip_token(p);
-   p = skip_token(p);
-   p = skip_token(p);
-   val.uint32 = strtol( p, (char **)NULL, 10 );
+   if (!EnumProcesses(aProcesses, sizeof(aProcesses), &cbNeeded)) {
+      cProcesses = 0;
+   } else {
+      cProcesses = cbNeeded / sizeof(DWORD);
+   }
+#if (_WIN32_WINNT >= 0x0501)
+   /* Only for XP or newer */
+   for (i = 0; i < cProcesses; i++)
+      if (aProcesses[i] != 0) {
+         hProcess = OpenProcess (PROCESS_QUERY_INFORMATION, 
+            FALSE, aProcesses[i]);
+         if (hProcess != NULL) {
+            if (IsProcessInJob(hProcess, NULL, &bResult)) {
+               if (bResult)
+                  running++;
+            }
+            CloseHandle(hProcess);
+         }
+      }
+#endif
 
-   if(val.uint32)
-     val.uint32--;/* don't count ourselves? */ 
+   val.uint32 = running;
 
    return val;
 }
 
+/* FIXME */
 g_val_t
 proc_total_func ( void )
 {
-   char *p;
+   DWORD aProcesses[MAXPROCESSES], cbNeeded, cProcesses;   
    g_val_t val;
 
-   p = update_file(&proc_loadavg);
-   p = skip_token(p);
-   p = skip_token(p);
-   p = skip_token(p); 
-   p = skip_whitespace(p);
-   while ( isdigit(*p) )
-      p++;
-   p++;  /* skip the slash-/ */ 
-   val.uint32 = strtol( p, (char **)NULL, 10 ); 
+   if (!EnumProcesses(aProcesses, sizeof(aProcesses), &cbNeeded)) {
+      cProcesses = 0;
+   } else {
+      cProcesses = cbNeeded / sizeof(DWORD);
+   }
+   val.uint32 = cProcesses; 
 
    return val;
 }
@@ -854,83 +775,72 @@ proc_total_func ( void )
 g_val_t
 mem_free_func ( void )
 {
-   char *p;
+   MEMORYSTATUSEX stat;
+   DWORDLONG size;
    g_val_t val;
 
-   p = strstr( update_file(&proc_meminfo), "MemFree:" );
-   if(p) {
-     p = skip_token(p);
-     val.uint32 = strtol( p, (char **)NULL, 10 );
+   stat.dwLength = sizeof(stat);
+
+   if ( GlobalMemoryStatusEx (&stat)) {
+      size = stat.ullAvailPhys;
+      /* get the value in kB */
+      val.f =  size / 1024;
    } else {
-     val.uint32 = 0;
+      val.f = 0;
    }
 
    return val;
 }
 
+/* FIXME */
 g_val_t
 mem_shared_func ( void )
 {
-   char *p;
    g_val_t val;
 
-   p = strstr( update_file(&proc_meminfo), "MemShared:" );
-   if (p) {
-      p = skip_token(p);
-      val.uint32 = strtol( p, (char **)NULL, 10 );
-   } else {
-      val.uint32 = 0;
-   }
+   val.f = 0;
 
    return val;
 }
 
+/* FIXME */
 g_val_t
 mem_buffers_func ( void )
 {
-   char *p;
    g_val_t val;
 
-   p = strstr( update_file(&proc_meminfo), "Buffers:" );
-   if(p) {
-     p = skip_token(p);
-     val.uint32 = strtol( p, (char **)NULL, 10 ); 
-   } else {
-     val.uint32 = 0;
-   }
+   val.f = 0;
 
    return val;
 }
 
+/* FIXME */
 g_val_t
 mem_cached_func ( void )
 {
-   char *p;
    g_val_t val;
 
-   p = strstr( update_file(&proc_meminfo), "Cached:");
-   if(p) {
-     p = skip_token(p);
-     val.uint32 = strtol( p, (char **)NULL, 10 );
-   } else {
-     val.uint32 = 0;
-   }
+   val.f = 0;
 
    return val;
 }
 
+/* FIXME: should be using PERFORMANCE_INFORMATION.CommitTotal */
 g_val_t
 swap_free_func ( void )
 {
-   char *p;
+   MEMORYSTATUSEX stat;
+   DWORDLONG size;
    g_val_t val;
 
-   p = strstr( update_file(&proc_meminfo), "SwapFree:" );
-   if(p) {
-     p = skip_token(p);
-     val.uint32 = strtol( p, (char **)NULL, 10 ); 
+   stat.dwLength = sizeof(stat);
+
+   if ( GlobalMemoryStatusEx (&stat)) {
+      size = stat.ullAvailPageFile;
+      /* get the value in kB */
+      val.f =  size / 1024;
    } else {
-     val.uint32 = 0;
+      val.f = 0;
    }
 
    return val;
@@ -949,148 +859,93 @@ mtu_func ( void )
    return val;
 }
 
-/* ===================================================== */
-/* Disk Usage. Using fsusage.c from the GNU fileutils-4.1.5 package. */
-#include <inttypes.h>
+/* FIXME: hardcoded max number of disks */
+#define MAXDRIVES 4
 
-/* Linux Specific, but we are in the Linux machine file. */
-#define MOUNTS "/proc/mounts"
+typedef struct {
+   double total;
+   double avail;
+} disk_t;
 
-struct nlist {
-   struct nlist *next;
-   char *name;
-};
-
-#define DFHASHSIZE 101
-static struct nlist *DFhashvector[DFHASHSIZE];
-
-/* --------------------------------------------------------------------------- */
-unsigned int DFhash(const char *s)
+static float
+find_disk_space(double *total, double *avail)
 {
-   unsigned int hashval;
-   for (hashval=0; *s != '\0'; s++)
-      hashval = *s + 31 * hashval;
-   return hashval % DFHASHSIZE;
-}
+   FILE *mnttab;
+   disk_t drives[MAXDRIVES];
+   struct mntent *ent;
+   struct statfs fs;
+   const double reported_units = 1e9;
+   int drive;
+   float pct;
+   float most_full = 0.0;
 
-/* --------------------------------------------------------------------------- */
-/* From K&R C book, pp. 144-145 */
-struct nlist * seen_before(const char *name)
-{
-   struct nlist *found=0, *np;
-   unsigned int hashval;
+   *total = 0.0;
+   *avail = 0.0;
+   bzero(drives, sizeof(disk_t) * MAXDRIVES);
 
-   /* lookup */
-   hashval=DFhash(name);
-   for (np=DFhashvector[hashval]; np; np=np->next) {
-      if (!strcmp(name,np->name)) {
-         found=np;
-         break;
+   mnttab = setmntent(MOUNTED, "r");
+   while ((ent = getmntent(mnttab)) != NULL) {
+      if (islower(ent->mnt_fsname[0])) {
+         drive = ent->mnt_fsname[0] - 'a';
+         if (drives[drive].total == 0.0) {
+            statfs(ent->mnt_fsname, &fs);
+         
+            drives[drive].total = (double)fs.f_blocks * fs.f_bsize;
+            drives[drive].avail = (double)fs.f_bavail * fs.f_bsize;
+         
+            pct = (drives[drive].avail == 0) ? 100.0 : 
+               ((drives[drive].total - 
+               drives[drive].avail)/drives[drive].total) * 100.0;
+
+            if (pct > most_full)
+               most_full = pct;
+
+            *total += (drives[drive].total / reported_units);
+            *avail += (drives[drive].avail / reported_units);
+         }
       }
    }
-   if (!found) {    /* not found */
-      np = (struct nlist *) malloc(sizeof(*np));
-      if (!np || !(np->name = (char *) strdup(name)))
-         return NULL;
-      np->next = DFhashvector[hashval];
-      DFhashvector[hashval] = np;
-      return NULL;
-   }
-   else /* found name */
-      return found;
-}
+   endmntent(mnttab);
 
-/* --------------------------------------------------------------------------- */
-void DFcleanup()
-{
-   struct nlist *np, *next;
-   int i;
-   for (i=0; i<DFHASHSIZE; i++) {
-      /* Non-standard for loop. Note the last clause happens at the end of the loop. */
-      for (np = DFhashvector[i]; np; np=next) {
-         next=np->next;
-         free(np->name);
-         free(np);
-      }
-      DFhashvector[i] = 0;
-   }
-}
+   return most_full;
+}   
 
 g_val_t
 disk_free_func( void )
 {
+   double total_free = 0.0;
+   double total_size = 0.0;
    g_val_t val;
-   val.d = 0;
+
+   find_disk_space(&total_size, &total_free); 
+
+   val.d = total_free;
    return val;
 }
 
 g_val_t
 disk_total_func( void )
 {
+   double total_free = 0.0;
+   double total_size = 0.0;
    g_val_t val;
-   val.d = 0;
+
+   find_disk_space(&total_size, &total_free);
+
+   val.d = total_size;
    return val;
 }
 
-/* --------------------------------------------------------------------------- */
 g_val_t
 part_max_used_func( void )
 {
+   double total_free = 0.0;
+   double total_size = 0.0;
+   float most_full;
    g_val_t val;
-   val.f = 0.0;
+
+   most_full = find_disk_space(&total_size, &total_free);
+
+   val.f = most_full;
    return val;
 }
-
-/* ---------------------------------- */
-/*
- * Stub for doing stand-alone testing
- *
- * Compile with:
- *
- *
-gcc -g -O2 -D_REENTRANT -Wall -I.. -I../lib -I../lib/dnet -DHAVE_CONFIG_H \
-   -DSTAND_ALONE_TEST  -o gmo.tst machine.c ../lib/.libs/libganglia.a \
-   ../lib/libgetopthelper.a -ldl -lresolv -lnsl -lpthread
- *
- */
-#ifdef STAND_ALONE_TEST
-int main()
-{
-g_val_t tval;
-
-metric_init();
-printf("num_cpustates = %d\n",num_cpustates);
-
-tval = proc_total_func();
-printf("Tot proc: %d\n",tval.uint32);
-
-cpu_user_func();
-cpu_nice_func();
-cpu_system_func();
-cpu_idle_func();
-cpu_wio_func();
-cpu_intr_func();
-cpu_sintr_func();
-
-
-sleep(5);
-
-tval = cpu_user_func();
-printf("User pct.: %f\n",tval.f);
-tval = cpu_nice_func();
-printf("Nice pct.: %f\n",tval.f);
-tval = cpu_system_func();
-printf("Sys  pct.: %f\n",tval.f);
-tval = cpu_idle_func();
-printf("Idle pct.: %f\n",tval.f);
-tval = cpu_wio_func();
-printf("Wio  pct.: %f\n",tval.f);
-tval = cpu_intr_func();
-printf("Intr pct.: %f\n",tval.f);
-tval = cpu_sintr_func();
-printf("Sntr pct.: %f\n",tval.f);
-
-return 0;
-}
-#endif
-
