@@ -10,9 +10,11 @@
 #include <sys/sysinfo.h>
 #include <sys/stat.h>
 #include <sys/statvfs.h>
+#include <sys/time.h>
 
 /* From old ganglia 2.5.x... */
 #include "file.h"
+#include "interface.h"
 #include "libmetrics.h"
 /* End old ganglia 2.5.x headers */
 
@@ -23,10 +25,10 @@
 typedef struct net_dev_stats net_dev_stats;
 struct net_dev_stats {
   char *name;
-  double pkts_in;
-  double pkts_out;
-  double bytes_in;
-  double bytes_out;
+  unsigned long rpi;
+  unsigned long rpo;
+  unsigned long rbi;
+  unsigned long rbo;
   net_dev_stats *next;
 };
 #define NHASH 101
@@ -35,34 +37,49 @@ static net_dev_stats *netstats[NHASH];
 
 /* Never changes */
 #ifndef BUFFSIZE
-#define BUFFSIZE 8192
+#define BUFFSIZE 16384
 #endif
 char proc_cpuinfo[BUFFSIZE];
 char proc_sys_kernel_osrelease[BUFFSIZE];
+
+#define SCALING_MAX_FREQ "/sys/devices/system/cpu/cpu0/cpufreq/scaling_max_freq"
 char sys_devices_system_cpu[32];
 int cpufreq;
 
 typedef struct {
-  uint32_t last_read;
-  uint32_t thresh;
+  struct timeval last_read;
+  float thresh;
   char *name;
   char buffer[BUFFSIZE];
 } timely_file;
 
-timely_file proc_stat    = { 0, 1, "/proc/stat" };
-timely_file proc_loadavg = { 0, 5, "/proc/loadavg" };
-timely_file proc_meminfo = { 0, 5, "/proc/meminfo" };
-timely_file proc_net_dev = { 0, 1, "/proc/net/dev" };
+timely_file proc_stat    = { {0,0} , 1., "/proc/stat" };
+timely_file proc_loadavg = { {0,0} , 5., "/proc/loadavg" };
+timely_file proc_meminfo = { {0,0} , 5., "/proc/meminfo" };
+timely_file proc_net_dev = { {0,0} , 1., "/proc/net/dev" };
+
+float timediff(const struct timeval *thistime, const struct timeval *lasttime)
+{
+  float diff;
+
+  diff = ((double) thistime->tv_sec * 1.0e6 +
+          (double) thistime->tv_usec -
+          (double) lasttime->tv_sec * 1.0e6 -
+          (double) lasttime->tv_usec) / 1.0e6;
+
+  return diff;
+}
 
 char *update_file(timely_file *tf)
 {
-  int now,rval;
-  now = time(0);
-  if(now - tf->last_read > tf->thresh) {
+  int rval;
+  struct timeval now;
+  gettimeofday(&now, NULL);
+  if(timediff(&now,&tf->last_read) > tf->thresh) {
     rval = slurpfile(tf->name, tf->buffer, BUFFSIZE);
     if(rval == SYNAPSE_FAILURE) {
       err_msg("update_file() got an error from slurpfile() reading %s",
-	      tf->name);
+              tf->name);
       return (char *)SYNAPSE_FAILURE;
     }
     else tf->last_read = now;
@@ -83,9 +100,11 @@ num_cpustates_func ( void )
    char *p;
    unsigned int i=0;
 
-   proc_stat.last_read=0;
+   proc_stat.last_read.tv_sec=0;
+   proc_stat.last_read.tv_usec=0;
    p = update_file(&proc_stat);
-   proc_stat.last_read=0;
+   proc_stat.last_read.tv_sec=0;
+   proc_stat.last_read.tv_usec=0;
 
 /*
 ** Skip initial "cpu" token
@@ -120,11 +139,7 @@ static unsigned int hashval(const char *s)
   return hval % NHASH;
 }
 
-static net_dev_stats *hash_lookup(char *devname, size_t nlen, const int enter, 
-				  const double pkts_in, 
-				  const double pkts_out, 
-				  const double bytes_in, 
-				  const double bytes_out)
+static net_dev_stats *hash_lookup(char *devname, size_t nlen) 
 {
   int hval;
   net_dev_stats *stats;
@@ -139,27 +154,160 @@ static net_dev_stats *hash_lookup(char *devname, size_t nlen, const int enter,
     }
   }
 
-  if (enter)
-  {
-    stats = (net_dev_stats *)malloc(sizeof(net_dev_stats));
-    if ( stats == NULL )
-    {   
-      err_msg("unable to allocate memory for /proc/net/dev/stats in hash_lookup()");
-      free(name);
-      return NULL;
-    }
-    stats->name = strndup(devname,nlen);
-    stats->pkts_in   = pkts_in;
-    stats->pkts_out  = pkts_out;
-    stats->bytes_in  = bytes_in;
-    stats->bytes_out = bytes_out;
-    stats->next = netstats[hval];
-    netstats[hval] = stats;
+  stats = (net_dev_stats *)malloc(sizeof(net_dev_stats));
+  if ( stats == NULL )
+  {   
+    err_msg("unable to allocate memory for /proc/net/dev/stats in hash_lookup(%s,%d)",name,nlen);
+    free(name);
+    return NULL;
   }
+  stats->name = strndup(devname,nlen);
+  stats->rpi = 0;
+  stats->rpo = 0;
+  stats->rbi = 0;
+  stats->rbo = 0;
+  stats->next = netstats[hval];
+  netstats[hval] = stats;
+
   free(name);
   return stats;
 }
 
+/*
+** FIXME: this routine should be rewritten to do per-interface statistics
+*/
+static double bytes_in=0,bytes_out=0,pkts_in=0,pkts_out=0;
+void update_ifdata ( char *caller )
+{
+   char *p;
+   int i;
+   static struct timeval stamp={0,0};
+   unsigned long rbi=0,rbo=0,rpi=0,rpo=0;
+   unsigned long l_bytes_in=0,l_bytes_out=0,l_pkts_in=0,l_pkts_out=0;
+   double l_bin,l_bout,l_pin,l_pout;
+
+   p = update_file(&proc_net_dev);
+   if ((proc_net_dev.last_read.tv_sec != stamp.tv_sec) &&
+       (proc_net_dev.last_read.tv_usec != stamp.tv_usec))
+      {
+	 /*  skip past the two-line header ... */
+	 p = index (p, '\n')+1;
+	 p = index (p, '\n')+1;
+
+	 while (*p != 0x00 )
+	    {
+	       /*  skip past the interface tag portion of this line */
+	       /*  but save the name of the interface (hash key) */
+	       char *src;
+	       size_t n = 0;
+
+	       while (p != 0x00 && isblank(*p))
+		  p++;
+	       src = p;
+	       while (p != 0x00 && *p != ':')
+		  {
+		     n++;
+		     p++;
+		  }
+
+	       p = index(p, ':');
+
+	       /* Ignore 'lo' and 'bond*' interfaces (but sanely) */
+	       if (p && strncmp (src, "lo", 2) &&
+		   strncmp (src, "bond", 4))
+		  { 
+		     p++;
+		     /* Check for data from the last read for this */
+		     /* interface.  If nothing exists, add to the table. */
+		     net_dev_stats *ns = hash_lookup(src, n); 
+		     if ( !ns ) return;
+
+		     /* receive */
+		     rbi = strtoul( p, &p ,10);
+		     if ( rbi >= ns->rbi ) {
+			l_bytes_in += rbi - ns->rbi;
+		     } else {
+			debug_msg("update_ifdata(%s) - Overflow in rbi: %lu -> %lu",caller,ns->rbi,rbi);
+			l_bytes_in += ULONG_MAX - ns->rbi + rbi;
+		     }
+		     ns->rbi = rbi;
+
+		     rpi = strtoul( p, &p ,10);
+		     if ( rpi >= ns->rpi ) {
+			l_pkts_in += rpi - ns->rpi;
+		     } else {
+			debug_msg("updata_ifdata(%s) - Overflow in rpi: %lu -> %lu",caller,ns->rpi,rpi);
+			l_pkts_in += ULONG_MAX - ns->rpi + rpi;
+		     }
+		     ns->rpi = rpi;
+		     
+		     /* skip unneeded metrics */
+		     for (i = 0; i < 6; i++) rbo = strtoul(p, &p, 10);
+
+		     /* transmit */
+		     rbo = strtoul( p, &p ,10);
+		     if ( rbo >= ns->rbo ) {
+			l_bytes_out += rbo - ns->rbo;
+		     } else {
+			debug_msg("update_ifdata(%s) - Overflow in rbo: %lu -> %lu",caller,ns->rbo,rbo);
+			l_bytes_out += ULONG_MAX - ns->rbo + rbo;
+		     }
+		     ns->rbo = rbo;
+
+		     rpo = strtoul( p, &p ,10);
+		     if ( rpo >= ns->rpo ) {
+			l_pkts_out += rpo - ns->rpo;
+		     } else {
+			debug_msg("update_ifdata(%s) - Overflow in rpo: %lu -> %lu",caller,ns->rpo,rpo);
+			l_pkts_out += ULONG_MAX - ns->rpo + rpo;
+		     }
+		     ns->rpo = rpo;
+		  }
+	       p = index (p, '\n') + 1;    // skips a line
+	    }
+
+	 /*
+	 ** Compute timediff. Check for bogus delta-t
+	 */
+	 float t = timediff(&proc_net_dev.last_read,&stamp);
+	 if ( t <  proc_net_dev.thresh) {
+	   err_msg("update_ifdata(%s) - Dubious delta-t: %f",caller,t);
+           return;
+	 }
+	 stamp = proc_net_dev.last_read;
+
+	 /*
+	 ** Compute rates in local variables
+	 */
+	 l_bin = l_bytes_in / t;
+	 l_bout = l_bytes_out / t;
+	 l_pin = l_pkts_in / t;
+	 l_pout = l_pkts_out / t;
+
+#ifdef REMOVE_BOGUS_SPIKES
+	 /*
+	 ** Check for "invalid" data, caused by HW error. Throw away dubious data points
+	 ** FIXME: This should be done per-interface, with threshholds depending on actual link speed
+	 */	
+	 if ((l_bin > 1.0e13) || (l_bout > 1.0e13) ||
+             (l_pin > 1.0e8)  || (l_pout > 1.0e8)) {
+		err_msg("update_ifdata(%s): %g %g %g %g / %g",caller,l_bin,l_bout,l_pin,l_pout,t);
+		return;
+	}
+#endif
+
+        /*
+        ** Finally return Values
+        */
+        bytes_in  = l_bin;
+        bytes_out = l_bout;
+        pkts_in   = l_pin;
+        pkts_out  = l_pout;
+      }
+
+   return;
+}
+
 /*
  * This function is called only once by the gmond.  Use to 
  * initialize data structures, etc or just return SYNAPSE_SUCCESS;
@@ -169,13 +317,16 @@ metric_init(void)
 {
    g_val_t rval;
    char * dummy;
+   struct stat struct_stat;
 
    num_cpustates = num_cpustates_func();
 
-   cpufreq = 1;
-   rval.int32 = slurpfile("/sys/devices/system/cpu/cpu0/cpufreq/scaling_max_freq", sys_devices_system_cpu, BUFFSIZE);
-   if ( rval.int32 == SYNAPSE_FAILURE )
-      cpufreq = 0;
+   /* scaling_max_freq will contain the max CPU speed if available */
+   cpufreq = 0;
+   if ( stat(SCALING_MAX_FREQ, &struct_stat) == 0 ) {
+      cpufreq = 1;
+      slurpfile(SCALING_MAX_FREQ, sys_devices_system_cpu, 32);
+   }
 
    rval.int32 = slurpfile("/proc/cpuinfo", proc_cpuinfo, BUFFSIZE);
    if ( rval.int32 == SYNAPSE_FAILURE )
@@ -199,10 +350,12 @@ metric_init(void)
    if ( dummy == (char *)SYNAPSE_FAILURE )
       {
          err_msg("metric_init() got an error from update_file()");
-	 rval.int32 = SYNAPSE_FAILURE;
+         rval.int32 = SYNAPSE_FAILURE;
          return rval;
       } 
 
+   update_ifdata("metric_inint");
+
    rval.int32 = SYNAPSE_SUCCESS;
    return rval;
 }
@@ -210,348 +363,44 @@ metric_init(void)
 g_val_t
 pkts_in_func ( void )
 {
-   char *p;
-   register int i;
-   static g_val_t val;
-   static int stamp;
-   double bytes_in=0, bytes_out=0, pkts_in=0, pkts_out=0, t = 0;
+   g_val_t val;
 
-   p = update_file(&proc_net_dev);
-   if (proc_net_dev.last_read != stamp)
-      {
-	 /*  skip past the two-line header ... */
-	 p = index (p, '\n')+1;
-	 p = index (p, '\n')+1;
-
-	 while (*p != 0x00 )
-	    {
-	       /*  skip past the interface tag portion of this line */
-	       /*  but save the name of the interface (hash key) */
-	       char *src;
-	       size_t n = 0;
-
-	       while (p != 0x00 && isblank(*p))
-		  p++;
-	       src = p;
-	       while (p != 0x00 && *p != ':')
-		  {
-		     n++;
-		     p++;
-		  }
-
-	       p = index(p, ':')+1;
-	       if ( ((*(p-2) != 'o') && (*(p-3) != 'l')) && 
-		    ((*(p-3) != 'd') && (*(p-4) != 'n') && 
-		     (*(p-5) != 'o') && (*(p-6) != 'b')) )
-		  { 
-		     double temp = 0.;
-		     /* Check for data from the last read for this */
-		     /* interface.  If nothing exists, add to the table. */
-		     net_dev_stats *ns = hash_lookup(src, n, 1, 
-						     0., 0., 0., 0.);
-		     if ( ns == NULL )
-		        {
-			   val.f = 0.;
-			   debug_msg(" **********  BYTES_OUT RETURN:  %f", 
-				     val.f);
-			   return val;
-			}
-
-		     t = strtod( p, &p ) - ns->bytes_in;
-		     if ( t < 0. ) { t += ULONG_MAX; }
-		     bytes_in += t;
-		     temp = strtod( p, &p );
-		     t = temp - ns->pkts_in;
-		     if ( t < 0. ) { t += ULONG_MAX; }
-		     pkts_in += t;
-		     for (i = 0; i < 6; i++) strtol(p, &p, 10);
-		     /* Fixed 2003 by Dr Michael Wirtz <m.wirtz@tinnit.de>
-			and Phil Radden <P.Radden@rl.ac.uk> */
-		     t = strtod( p, &p ) - ns->bytes_out;
-		     if ( t < 0. ) { t += ULONG_MAX; }
-		     bytes_out += t;
-		     t = strtod( p, &p ) - ns->pkts_out;
-		     if ( t < 0. ) { t += ULONG_MAX; }
-		     pkts_out += t;
-
-		     /* Update the hash for this measurement */
-		     ns->pkts_in = temp;
-		  }
-	       p = index (p, '\n') + 1;    // skips a line
-	    }
-
-	 val.f = 0.;
-	 if ( pkts_in >= 1. )
-	    {
-	       t = proc_net_dev.last_read - stamp;
-	       val.f = pkts_in / t;
-	    }
-
-	 stamp = proc_net_dev.last_read;
-      }
-
-   debug_msg(" **********  PKTS_IN RETURN:  %f", val.f);
+   update_ifdata("PI");
+   val.f = pkts_in;
+   debug_msg(" ********** pkts_in:  %f", pkts_in);
    return val;
 }
 
 g_val_t
 pkts_out_func ( void )
 {
-   char *p;
-   register int i;
-   static g_val_t val;
-   static int stamp;
-   double bytes_in=0, bytes_out=0, pkts_in=0, pkts_out=0, t = 0;
+   g_val_t val;
 
-   p = update_file(&proc_net_dev);
-   if (proc_net_dev.last_read != stamp)
-      {
-	 /*  skip past the two-line header ... */
-	 p = index (p, '\n')+1;
-	 p = index (p, '\n')+1;
-
-	 while (*p != 0x00 )
-	    {
-	       /*  skip past the interface tag portion of this line */
-	       /*  but save the name of the interface (hash key) */
-	       char *src;
-	       size_t n = 0;
-
-	       while (p != 0x00 && isblank(*p))
-		  p++;
-	       src = p;
-	       while (p != 0x00 && *p != ':')
-		  {
-		     n++;
-		     p++;
-		  }
-
-	       p = index(p, ':')+1;
-	       if ( ((*(p-2) != 'o') && (*(p-3) != 'l')) && 
-		    ((*(p-3) != 'd') && (*(p-4) != 'n') && 
-		     (*(p-5) != 'o') && (*(p-6) != 'b')) )
-		  { 
-		     double temp = 0.;
-		     /* Check for data from the last read for this */
-		     /* interface.  If nothing exists, add to the table. */
-		     net_dev_stats *ns = hash_lookup(src, n, 1, 
-						     0., 0., 0., 0.);
-		     if ( ns == NULL )
-		        {
-			   val.f = 0.;
-			   debug_msg(" **********  BYTES_OUT RETURN:  %f", 
-				     val.f);
-			   return val;
-			}
-
-		     t = strtod( p, &p ) - ns->bytes_in;
-		     if ( t < 0. ) { t += ULONG_MAX; }
-		     bytes_in += t;
-		     t = strtod( p, &p ) - ns->pkts_in;
-		     if ( t < 0. ) { t += ULONG_MAX; }
-		     pkts_in += t;
-		     for (i = 0; i < 6; i++) strtol(p, &p, 10);
-		     /* Fixed 2003 by Dr Michael Wirtz <m.wirtz@tinnit.de>
-			and Phil Radden <P.Radden@rl.ac.uk> */
-		     t = strtod( p, &p ) - ns->bytes_out;
-		     if ( t < 0. ) { t += ULONG_MAX; }
-		     bytes_out += t;
-		     temp = strtod( p, &p );
-		     t = temp - ns->pkts_out;
-		     if ( t < 0. ) { t += ULONG_MAX; }
-		     pkts_out += t;
-
-		     /* Update the hash for this measurement */
-		     ns->pkts_out = temp;
-		  }
-	       p = index (p, '\n') + 1;    // skips a line
-	    }
-
-	 val.f = 0.;
-	 if ( pkts_out >= 1. )
-	    {
-	       t = proc_net_dev.last_read - stamp;
-	       val.f = pkts_out / t;
-	    }
-
-	 stamp = proc_net_dev.last_read;
-      }
-
-   debug_msg(" **********  PKTS_OUT RETURN:  %f", val.f);
+   update_ifdata("PO");
+   val.f = pkts_out;
+   debug_msg(" ********** pkts_out:  %f", pkts_out);
    return val;
 }
 
 g_val_t
 bytes_out_func ( void )
 {
-   char *p;
-   register int i;
-   static g_val_t val;
-   static int stamp;
-   double bytes_in=0, bytes_out=0, pkts_in=0, pkts_out=0, t = 0;
+   g_val_t val;
 
-   p = update_file(&proc_net_dev);
-   if (proc_net_dev.last_read != stamp)
-      {
-	 /*  skip past the two-line header ... */
-	 p = index (p, '\n')+1;
-	 p = index (p, '\n')+1;
-
-	 while (*p != 0x00 )
-	    {
-	       /*  skip past the interface tag portion of this line */
-	       /*  but save the name of the interface (hash key) */
-	       char *src;
-	       size_t n = 0;
-
-	       while (p != 0x00 && isblank(*p))
-		  p++;
-	       src = p;
-	       while (p != 0x00 && *p != ':')
-		  {
-		     n++;
-		     p++;
-		  }
-
-	       p = index(p, ':')+1;
-	       if ( ((*(p-2) != 'o') && (*(p-3) != 'l')) && 
-		    ((*(p-3) != 'd') && (*(p-4) != 'n') && 
-		     (*(p-5) != 'o') && (*(p-6) != 'b')) )
-		  { 
-		     double temp = 0.;
-		     /* Check for data from the last read for this */
-		     /* interface.  If nothing exists, add to the table. */
-		     net_dev_stats *ns = hash_lookup(src, n, 1, 
-						     0., 0., 0., 0.);
-		     if ( ns == NULL )
-		        {
-			   val.f = 0.;
-			   debug_msg(" **********  BYTES_OUT RETURN:  %f", 
-				     val.f);
-			   return val;
-			}
-
-		     t = strtod( p, &p ) - ns->bytes_in;
-		     if ( t < 0. ) { t += ULONG_MAX; }
-		     bytes_in += t;
-		     t = strtod( p, &p ) - ns->pkts_in;
-		     if ( t < 0. ) { t += ULONG_MAX; }
-		     pkts_in += t;
-		     for (i = 0; i < 6; i++) strtol(p, &p, 10);
-		     /* Fixed 2003 by Dr Michael Wirtz <m.wirtz@tinnit.de>
-			and Phil Radden <P.Radden@rl.ac.uk> */
-		     temp = strtod( p, &p );
-		     t = temp - ns->bytes_out;
-		     if ( t < 0. ) { t += ULONG_MAX; }
-		     bytes_out += t;
-		     t = strtod( p, &p ) - ns->pkts_out;
-		     if ( t < 0. ) { t += ULONG_MAX; }
-		     pkts_out += t;
-
-		     /* Update the hash for this measurement */
-		     ns->bytes_out = temp;
-		  }
-	       p = index (p, '\n') + 1;    // skips a line
-	    }
-
-	 val.f = 0.;
-	 if ( bytes_out >= 1. )
-	    {
-	       t = proc_net_dev.last_read - stamp;
-	       val.f = bytes_out / t;
-	    }
-
-	 stamp = proc_net_dev.last_read;
-      }
-
-   debug_msg(" **********  BYTES_OUT RETURN:  %f", val.f);
+   update_ifdata("BO");
+   val.f = bytes_out;
+   debug_msg(" ********** bytes_out:  %f", bytes_out);
    return val;
 }
 
 g_val_t
 bytes_in_func ( void )
 {
-   char *p;
-   register int i;
-   static g_val_t val;
-   static int stamp;
-   double bytes_in=0, bytes_out=0, pkts_in=0, pkts_out=0, t = 0;
+   g_val_t val;
 
-   p = update_file(&proc_net_dev);
-   if (proc_net_dev.last_read != stamp)
-      {
-	 /*  skip past the two-line header ... */
-	 p = index (p, '\n')+1;
-	 p = index (p, '\n')+1;
-
-	 while (*p != 0x00 )
-	    {
-	       /*  skip past the interface tag portion of this line */
-	       /*  but save the name of the interface (hash key) */
-	       char *src;
-	       size_t n = 0;
-
-	       while (p != 0x00 && isblank(*p))
-		  p++;
-	       src = p;
-	       while (p != 0x00 && *p != ':')
-		  {
-		     n++;
-		     p++;
-		  }
-
-	       p = index(p, ':')+1;
-	       if ( ((*(p-2) != 'o') && (*(p-3) != 'l')) && 
-		    ((*(p-3) != 'd') && (*(p-4) != 'n') && 
-		     (*(p-5) != 'o') && (*(p-6) != 'b')) )
-		  { 
-		     double temp = 0.;
-		     /* Check for data from the last read for this */
-		     /* interface.  If nothing exists, add to the table. */
-		     net_dev_stats *ns = hash_lookup(src, n, 1, 
-						     0., 0., 0., 0.);
-		     if ( ns == NULL )
-		        {
-			   val.f = 0.;
-			   debug_msg(" **********  BYTES_OUT RETURN:  %f", 
-				     val.f);
-			   return val;
-			}
-
-		     temp = strtod( p, &p );
-		     t = temp - ns->bytes_in;
-		     if ( t < 0. ) { t += ULONG_MAX; }
-		     bytes_in += t;
-		     t = strtod( p, &p ) - ns->pkts_in;
-		     if ( t < 0. ) { t += ULONG_MAX; }
-		     pkts_in += t;
-		     for (i = 0; i < 6; i++) strtol(p, &p, 10);
-		     /* Fixed 2003 by Dr Michael Wirtz <m.wirtz@tinnit.de>
-			and Phil Radden <P.Radden@rl.ac.uk> */
-		     t = strtod( p, &p ) - ns->bytes_out;
-		     if ( t < 0. ) { t += ULONG_MAX; }
-		     bytes_out += t;
-		     t = strtod( p, &p ) - ns->pkts_out;
-		     if ( t < 0. ) { t += ULONG_MAX; }
-		     pkts_out += t;
-
-		     /* Update the hash for this measurement */
-		     ns->bytes_in = temp;
-		  }
-	       p = index (p, '\n') + 1;    // skips a line
-	    }
-
-	 val.f = 0.;
-	 if ( bytes_in >= 1. )
-	    {
-	       t = proc_net_dev.last_read - stamp;
-	       val.f = bytes_in / t;
-	    }
-
-	 stamp = proc_net_dev.last_read;
-      }
-
-   debug_msg(" **********  BYTES_IN RETURN:  %f", val.f);
+   update_ifdata("BI");
+   val.f = bytes_in;
+   debug_msg(" ********** bytes_in:  %f", bytes_in);
    return val;
 }
 
@@ -635,6 +484,9 @@ cpu_speed_func ( void )
    return val;
 }
 
+/*
+** FIXME: all functions using /proc/meminfo sould use a central routine like networking
+*/
 g_val_t
 mem_total_func ( void )
 {
@@ -644,9 +496,9 @@ mem_total_func ( void )
    p = strstr( update_file(&proc_meminfo), "MemTotal:");
    if(p) {
      p = skip_token(p);
-     val.uint32 = strtol( p, (char **)NULL, 10 );
+     val.f = atof( p );
    } else {
-     val.uint32 = 0;
+     val.f = 0;
    }
 
    return val;
@@ -661,9 +513,9 @@ swap_total_func ( void )
    p = strstr( update_file(&proc_meminfo), "SwapTotal:" );
    if(p) {
      p = skip_token(p);
-     val.uint32 = strtol( p, (char **)NULL, 10 );  
+     val.f = atof( p );  
    } else {
-     val.uint32 = 0;
+     val.f = 0;
    }
 
    return val;
@@ -680,7 +532,7 @@ boottime_func ( void )
    p = strstr ( p, "btime" );
    if(p) { 
      p = skip_token ( p );
-     val.uint32 = strtod ( p, (char **)NULL );
+     val.uint32 = atoi ( p );
    } else {
      val.uint32 = 0;
    }
@@ -757,6 +609,9 @@ os_release_func ( void )
    return val;
 }
 
+/*
+** FIXME: all functions using /proc/stat should be rewritten to use a central function like networking
+*/
 
 /*
  * A helper function to return the total number of cpu jiffies
@@ -799,12 +654,13 @@ cpu_user_func ( void )
 {
    char *p;
    static g_val_t val;
-   static int stamp;
+   static struct timeval stamp={0,0};
    static double last_user_jiffies,  user_jiffies, 
                  last_total_jiffies, total_jiffies, diff;
    
    p = update_file(&proc_stat);
-   if(proc_stat.last_read != stamp) {
+   if((proc_stat.last_read.tv_sec != stamp.tv_sec) &&
+      (proc_stat.last_read.tv_usec != stamp.tv_usec)) {
      stamp = proc_stat.last_read;
      
      p = skip_token(p);
@@ -830,12 +686,13 @@ cpu_nice_func ( void )
 {
    char *p;
    static g_val_t val;
-   static int stamp;
+   static struct timeval stamp={0,0};
    static double last_nice_jiffies,  nice_jiffies,
                  last_total_jiffies, total_jiffies, diff;
  
    p = update_file(&proc_stat);
-   if(proc_stat.last_read != stamp) {
+   if((proc_stat.last_read.tv_sec != stamp.tv_sec) &&
+      (proc_stat.last_read.tv_usec != stamp.tv_usec)) {
      stamp = proc_stat.last_read;
  
      p = skip_token(p);
@@ -862,12 +719,13 @@ cpu_system_func ( void )
 {
    char *p;
    static g_val_t val;
-   static int stamp;
+   static struct timeval stamp={0,0};
    static double last_system_jiffies,  system_jiffies,
                  last_total_jiffies, total_jiffies, diff;
  
    p = update_file(&proc_stat);
-   if(proc_stat.last_read != stamp) {
+   if((proc_stat.last_read.tv_sec != stamp.tv_sec) &&
+      (proc_stat.last_read.tv_usec != stamp.tv_usec)) {
      stamp = proc_stat.last_read;
      
      p = skip_token(p);
@@ -903,12 +761,13 @@ cpu_idle_func ( void )
 {
    char *p;
    static g_val_t val;
-   static int stamp;
+   static struct timeval stamp={0,0};
    static double last_idle_jiffies,  idle_jiffies,
                  last_total_jiffies, total_jiffies, diff;
  
    p = update_file(&proc_stat);
-   if(proc_stat.last_read != stamp) {
+   if((proc_stat.last_read.tv_sec != stamp.tv_sec) &&
+      (proc_stat.last_read.tv_usec != stamp.tv_usec)) {
      stamp = proc_stat.last_read;
      
      p = skip_token(p);
@@ -958,7 +817,7 @@ cpu_wio_func ( void )
 {
    char *p;
    static g_val_t val;
-   static int stamp;
+   static struct timeval stamp={0,0};
    static double last_wio_jiffies,  wio_jiffies,
                  last_total_jiffies, total_jiffies, diff;
  
@@ -968,7 +827,8 @@ cpu_wio_func ( void )
      }
 
    p = update_file(&proc_stat);
-   if(proc_stat.last_read != stamp) {
+   if((proc_stat.last_read.tv_sec != stamp.tv_sec) &&
+      (proc_stat.last_read.tv_usec != stamp.tv_usec)) {
      stamp = proc_stat.last_read;
      
      p = skip_token(p);
@@ -999,7 +859,7 @@ cpu_intr_func ( void )
 {
    char *p;
    static g_val_t val;
-   static int stamp;
+   static struct timeval stamp={0,0};
    static double last_intr_jiffies,  intr_jiffies,
                  last_total_jiffies, total_jiffies, diff;
  
@@ -1009,7 +869,8 @@ cpu_intr_func ( void )
      }
 
    p = update_file(&proc_stat);
-   if(proc_stat.last_read != stamp) {
+   if((proc_stat.last_read.tv_sec != stamp.tv_sec) &&
+      (proc_stat.last_read.tv_usec != stamp.tv_usec)) {
      stamp = proc_stat.last_read;
      
      p = skip_token(p);
@@ -1041,7 +902,7 @@ cpu_sintr_func ( void )
 {
    char *p;
    static g_val_t val;
-   static int stamp;
+   static struct timeval stamp={0,0};
    static double last_sintr_jiffies,  sintr_jiffies,
                  last_total_jiffies, total_jiffies, diff;
  
@@ -1051,7 +912,8 @@ cpu_sintr_func ( void )
      }
 
    p = update_file(&proc_stat);
-   if(proc_stat.last_read != stamp) {
+   if((proc_stat.last_read.tv_sec != stamp.tv_sec) &&
+      (proc_stat.last_read.tv_usec != stamp.tv_usec)) {
      stamp = proc_stat.last_read;
      
      p = skip_token(p);
@@ -1165,9 +1027,9 @@ mem_free_func ( void )
    p = strstr( update_file(&proc_meminfo), "MemFree:" );
    if(p) {
      p = skip_token(p);
-     val.uint32 = strtol( p, (char **)NULL, 10 );
+     val.f = atof( p );
    } else {
-     val.uint32 = 0;
+     val.f = 0;
    }
 
    return val;
@@ -1185,9 +1047,9 @@ mem_shared_func ( void )
    p = strstr( update_file(&proc_meminfo), "MemShared:" );
    if (p) {
       p = skip_token(p);
-      val.uint32 = strtol( p, (char **)NULL, 10 );
+      val.f = atof( p );
    } else {
-      val.uint32 = 0;
+      val.f = 0;
    }
 
    return val;
@@ -1202,9 +1064,9 @@ mem_buffers_func ( void )
    p = strstr( update_file(&proc_meminfo), "Buffers:" );
    if(p) {
      p = skip_token(p);
-     val.uint32 = strtol( p, (char **)NULL, 10 ); 
+     val.f = atof( p ); 
    } else {
-     val.uint32 = 0;
+     val.f = 0;
    }
 
    return val;
@@ -1219,9 +1081,9 @@ mem_cached_func ( void )
    p = strstr( update_file(&proc_meminfo), "Cached:");
    if(p) {
      p = skip_token(p);
-     val.uint32 = strtol( p, (char **)NULL, 10 );
+     val.f = atof( p );
    } else {
-     val.uint32 = 0;
+     val.f = 0;
    }
 
    return val;
@@ -1236,9 +1098,9 @@ swap_free_func ( void )
    p = strstr( update_file(&proc_meminfo), "SwapFree:" );
    if(p) {
      p = skip_token(p);
-     val.uint32 = strtol( p, (char **)NULL, 10 ); 
+     val.f = atof( p ); 
    } else {
-     val.uint32 = 0;
+     val.f = 0;
    }
 
    return val;
@@ -1257,10 +1119,6 @@ mtu_func ( void )
    return val;
 }
 
-/* ===================================================== */
-/* Disk Usage. Using fsusage.c from the GNU fileutils-4.1.5 package. */
-#include <inttypes.h>
-
 /* Linux Specific, but we are in the Linux machine file. */
 #define MOUNTS "/proc/mounts"
 
@@ -1448,57 +1306,3 @@ part_max_used_func( void )
    val.f = most_full;
    return val;
 }
-
-/* ---------------------------------- */
-/*
- * Stub for doing stand-alone testing
- *
- * Compile with:
- *
- *
-gcc -g -O2 -D_REENTRANT -Wall -I.. -I../lib -I../lib/dnet -DHAVE_CONFIG_H \
-   -DSTAND_ALONE_TEST  -o gmo.tst machine.c ../lib/.libs/libganglia.a \
-   ../lib/libgetopthelper.a -ldl -lresolv -lnsl -lpthread
- *
- */
-#ifdef STAND_ALONE_TEST
-int main()
-{
-g_val_t tval;
-
-metric_init();
-printf("num_cpustates = %d\n",num_cpustates);
-
-tval = proc_total_func();
-printf("Tot proc: %d\n",tval.uint32);
-
-cpu_user_func();
-cpu_nice_func();
-cpu_system_func();
-cpu_idle_func();
-cpu_wio_func();
-cpu_intr_func();
-cpu_sintr_func();
-
-
-sleep(5);
-
-tval = cpu_user_func();
-printf("User pct.: %f\n",tval.f);
-tval = cpu_nice_func();
-printf("Nice pct.: %f\n",tval.f);
-tval = cpu_system_func();
-printf("Sys  pct.: %f\n",tval.f);
-tval = cpu_idle_func();
-printf("Idle pct.: %f\n",tval.f);
-tval = cpu_wio_func();
-printf("Wio  pct.: %f\n",tval.f);
-tval = cpu_intr_func();
-printf("Intr pct.: %f\n",tval.f);
-tval = cpu_sintr_func();
-printf("Sntr pct.: %f\n",tval.f);
-
-return 0;
-}
-#endif
-
