@@ -3,6 +3,10 @@
  *  by Jan Schaumann <jschauma@netbsd.org>
  *  Thu Feb  3 22:33:36 EST 2005
  *
+ *  Tested on NetBSD 4.0 (i386)
+ *  Tested on NetBSD 3.1 (amd64 and i386)
+ *  Tested on NetBSD 3.0 (i386)
+ *  Tested on NetBSD 2.0.2 (i386)
  */
 
 #include <kvm.h>
@@ -34,6 +38,9 @@
 #include <fcntl.h>
 #include <limits.h>
 #include <paths.h>
+#include <string.h>
+#include <strings.h>
+#include <stdlib.h>
 
 #include "interface.h"
 #include "libmetrics.h"
@@ -71,12 +78,10 @@ static int	checkvfsname(const char *, const char **);
 static const char **makevfslist(char *);
 static float	find_disk_space(double *, double *);
 
+static int	cpu_state(int which);
 // XXX
 //static int	vm_state(int);
 
-static int 	use_vm_swap_info = 0;
-static int 	mibswap[MIB_SWAPINFO_SIZE];
-static size_t 	mibswap_size;
 static kvm_t 	*kd = NULL;
 static int 	pagesize;
 static int	skipvfs;
@@ -131,7 +136,7 @@ cpu_speed_func ( void )
 
 #if (__NetBSD_Version__ > 299000000)
    if (sysctlbyname("machdep.est.frequency.target", &cpu_speed, &len, NULL, 0) == -1)
-	val.uint32 = 0;
+      val.uint32 = 0;
 #endif
 
    val.uint32 = cpu_speed /= 1000000;
@@ -152,7 +157,7 @@ mem_total_func ( void )
 
    sysctl(mib, 2, &total, &len, NULL, 0);
    total /= 1024;
-   val.uint32 = total;
+   val.f = total;
    return val;
 }
 
@@ -166,7 +171,7 @@ swap_total_func ( void )
 	size_t len;
 	int totswap;
 
-	val.uint32 = 0;
+	val.f = 0;
 	totswap = 0;
 
 	mib[0] = CTL_VM;
@@ -180,7 +185,7 @@ swap_total_func ( void )
 	else
 		totswap = uvmexp.swpages;
 
-	val.uint32 = totswap * (pagesize / 1024);
+	val.f = totswap * (pagesize / 1024);
 
 	return val;
 }
@@ -281,7 +286,6 @@ cpu_state(int which)
 	static struct timeval this_time, last_time;
 	struct timeval time_diff;
 	size_t len = sizeof(cp_time);
-	int i;
 
 	if (which == -1) {
 		bzero(cp_old, sizeof(cp_old));
@@ -332,9 +336,9 @@ cpu_user_func ( void )
    if (sysctl(mib, 2, &cp_time, &len, NULL, 0) == -1 || !len)
       res = 0.0;
    else {
-	   /* Use percentages function lifted from top(1) to figure percentages */
-	   tot = percentages(CPUSTATES, cpu_states, cp_time, cp_old, cp_diff);
-	   res = cpu_states[CP_USER];
+      /* Use percentages function lifted from top(1) to figure percentages */
+      tot = percentages(CPUSTATES, cpu_states, cp_time, cp_old, cp_diff);
+      res = cpu_states[CP_USER];
    }
    val.f = (float)res/10;
    return val;
@@ -363,9 +367,9 @@ cpu_nice_func ( void )
    if (sysctl(mib, 2, &cp_time, &len, NULL, 0) == -1 || !len)
       res = 0.0;
    else {
-	   /* Use percentages function lifted from top(1) to figure percentages */
-	   tot = percentages(CPUSTATES, cpu_states, cp_time, cp_old, cp_diff);
-	   res = cpu_states[CP_NICE];
+      /* Use percentages function lifted from top(1) to figure percentages */
+      tot = percentages(CPUSTATES, cpu_states, cp_time, cp_old, cp_diff);
+      res = cpu_states[CP_NICE];
    }
    val.f = (float)res/10;
    return val;
@@ -394,9 +398,9 @@ cpu_system_func ( void )
    if (sysctl(mib, 2, &cp_time, &len, NULL, 0) == -1 || !len)
       res = 0.0;
    else {
-	   /* Use percentages function lifted from top(1) to figure percentages */
-	   tot = percentages(CPUSTATES, cpu_states, cp_time, cp_old, cp_diff);
-	   res = cpu_states[CP_SYS];
+      /* Use percentages function lifted from top(1) to figure percentages */
+      tot = percentages(CPUSTATES, cpu_states, cp_time, cp_old, cp_diff);
+      res = cpu_states[CP_SYS];
    }
    val.f = (float)res/10;
    return val;
@@ -425,9 +429,9 @@ cpu_idle_func ( void )
    if (sysctl(mib, 2, &cp_time, &len, NULL, 0) == -1 || !len)
       res = 0.0;
    else {
-	   /* Use percentages function lifted from top(1) to figure percentages */
-	   tot = percentages(CPUSTATES, cpu_states, cp_time, cp_old, cp_diff);
-	   res = cpu_states[CP_IDLE];
+      /* Use percentages function lifted from top(1) to figure percentages */
+      tot = percentages(CPUSTATES, cpu_states, cp_time, cp_old, cp_diff);
+      res = cpu_states[CP_IDLE];
    }
    val.f = (float)res/10;
    return val;
@@ -476,9 +480,9 @@ cpu_intr_func ( void )
    if (sysctl(mib, 2, &cp_time, &len, NULL, 0) == -1 || !len)
       res = 0.0;
    else {
-	   /* Use percentages function lifted from top(1) to figure percentages */
-	   tot = percentages(CPUSTATES, cpu_states, cp_time, cp_old, cp_diff);
-	   res = cpu_states[CP_INTR];
+      /* Use percentages function lifted from top(1) to figure percentages */
+      tot = percentages(CPUSTATES, cpu_states, cp_time, cp_old, cp_diff);
+      res = cpu_states[CP_INTR];
    }
 
    val.f = (float)res/10;
@@ -580,19 +584,19 @@ proc_run_func( void )
 #endif
       switch(state) {
 #if (__NetBSD_Version__ >= 200000000)
-	 case SACTIVE:
+         case SACTIVE:
 #else
-	 case SRUN:
-	 case SONPROC:
+         case SRUN:
+         case SONPROC:
 #endif
-	 case SIDL:
-		val.uint32++;
-		break;
+         case SIDL:
+            val.uint32++;
+            break;
       }
    }
 
    if (val.uint32 > 0)
-	val.uint32--;
+      val.uint32--;
 
 output:
    return val;
@@ -610,9 +614,9 @@ mem_free_func ( void )
 	mib[0] = CTL_VM;
 	mib[1] = VM_METER;
 	if (sysctl(mib, 2, &vminfo, &len, NULL, 0) < 0)
-		val.uint32 = 0;
+		val.f = 0;
 	else
-		val.uint32 = vminfo.t_free * (pagesize / 1024);
+		val.f = vminfo.t_free * (pagesize / 1024);
 	return val;
 }
 
@@ -628,9 +632,9 @@ mem_shared_func ( void )
 	mib[0] = CTL_VM;
 	mib[1] = VM_METER;
 	if (sysctl(mib, 2, &vminfo, &len, NULL, 0) < 0)
-		val.uint32 = 0;
+		val.f = 0;
 	else
-		val.uint32 = (vminfo.t_vmshr + vminfo.t_rmshr) * (pagesize / 1024);
+		val.f = (vminfo.t_vmshr + vminfo.t_rmshr) * (pagesize / 1024);
 	return val;
 }
 
@@ -646,11 +650,13 @@ mem_buffers_func ( void )
    mib[1] = VM_NKMEMPAGES;
 
    len = sizeof (buffers);
-   if((sysctl(mib, 2, &buffers, &len, NULL, 0) == -1) || !len)
-		buffers = 0; 
-   buffers /= 1024;
 
-   val.uint32 = buffers;
+   if((sysctl(mib, 2, &buffers, &len, NULL, 0) == -1) || !len)
+      buffers = 0; 
+
+   buffers /= 1024;
+   val.f = buffers;
+
    return val;
 }
 
@@ -674,7 +680,7 @@ mem_cached_func ( void )
 	else
 		cache = uvmexp.execpages + uvmexp.filepages;
 
-	val.uint32 = cache * (pagesize / 1024);
+	val.f = cache * (pagesize / 1024);
 	return val;
 }
 
@@ -688,7 +694,7 @@ swap_free_func ( void )
 	size_t len;
 	int totswapf;
 
-	val.uint32 = 0;
+	val.f = 0;
 	totswapf = 0;
 
 	mib[0] = CTL_VM;
@@ -702,7 +708,7 @@ swap_free_func ( void )
 	else
 		totswapf = uvmexp.swpages - uvmexp.swpginuse;
 
-	val.uint32 = totswapf * (pagesize / 1024);
+	val.f = totswapf * (pagesize / 1024);
 
 	return val;
 }
@@ -924,7 +930,7 @@ find_disk_space(double *total, double *tot_avail)
 	*total = 0.0;
 	*tot_avail = 0.0;
 
-	fstype = "ufs";
+	fstype = "ffs";
 
 	netvfslist = makenetvfslist();
 	vfslist = makevfslist(netvfslist);
@@ -933,7 +939,8 @@ find_disk_space(double *total, double *tot_avail)
 	mntsize = getmntinfo(&mntbuf, MNT_NOWAIT);
 	mntsize = regetmntinfo(&mntbuf, mntsize, vfslist);
 	for (i = 0; i < mntsize; i++) {
-		if ((mntbuf[i].f_flags & MNT_IGNORE) == 0) {
+		if (((mntbuf[i].f_flags & MNT_IGNORE) == 0) && 
+		    (strcmp(mntbuf[i].f_fstypename, fstype) == 0)) {
 			used = mntbuf[i].f_blocks - mntbuf[i].f_bfree;
 			availblks = mntbuf[i].f_bavail + used;
 			pct = (availblks == 0 ? 100.0 :
@@ -1028,7 +1035,6 @@ makenetvfslist(void)
 
 	int mib[3], maxvfsconf;
 	size_t miblen;
-	struct ovfsconf *ptr;
 
 	mib[0] = CTL_VFS; mib[1] = VFS_GENERIC; mib[2] = VFS_MAXTYPENUM;
 	miblen=sizeof(maxvfsconf);
@@ -1044,16 +1050,6 @@ makenetvfslist(void)
 	}
 
 	cnt = 0;
-	/* XXX: use sysctl ? */
-	/* while ((ptr = getvfsent()) != NULL && cnt < maxvfsconf) {
-
-		listptr[cnt] = strdup(ptr->vfc_name);
-		if (listptr[cnt] == NULL) {
-			warnx("malloc failed");
-			goto done;
-		}
-		cnt++;
-	} */
 	if (cnt == 0)
 		goto done;
 	/*
@@ -1268,14 +1264,14 @@ counterdiff(uint64_t oldval, uint64_t newval, uint64_t maxval, uint64_t maxdiff)
 		return (newval - oldval);
 
 	/*
-         * Now the tricky part.  If we assume counters never get reset,
-         * this is easy.  Unfortunaly, they do get reset on some
-         * systems, so we need to try and deal with that.  Our huristic
-         * is that if out difference is greater then maxdiff and newval
-         * is less or equal to maxdiff, then we've probably been reset
-         * rather then actually wrapping.  Obviously, you need to be
-         * careful to poll often enough that you won't exceed maxdiff or
-         * you will get undersized numbers when you do wrap.
+	 * Now the tricky part.  If we assume counters never get reset,
+	 * this is easy.  Unfortunaly, they do get reset on some
+	 * systems, so we need to try and deal with that.  Our huristic
+	 * is that if out difference is greater then maxdiff and newval
+	 * is less or equal to maxdiff, then we've probably been reset
+	 * rather then actually wrapping.  Obviously, you need to be
+	 * careful to poll often enough that you won't exceed maxdiff or
+	 * you will get undersized numbers when you do wrap.
 	 */
 	diff = maxval - oldval + newval;
 	if (diff > maxdiff && newval <= maxdiff)
