@@ -174,7 +174,7 @@ struct g_metrics_struct {
 
 static struct g_metrics_struct metriclist;
 static kstat_ctl_t *kc=NULL;
-static int ncpus;
+static int g_ncpus;
 
 int
 get_kstat_val(g_val_t *val, char *km_name, char *ks_name, char *name)
@@ -203,13 +203,16 @@ get_kstat_val(g_val_t *val, char *km_name, char *ks_name, char *name)
    ks = kstat_lookup(kc, km_name, 0, ks_name);
    debug_msg("%s: Just did kstat_lookup().",name);
 
-   /* CPU_INFO module & instance check: Michael Hom <michael_hom_work@yahoo.com>
-    * If the first online CPU is not in slot #0, gmond will segfault and core dump.
-    * If "km_name = NULL" and "km_instance = -1", then first instance of the module is returned.
-    *     ks = kstat_lookup(kc, km_name, -1, NULL);
+   /*
+    * A hack contributed by Michael Hom <michael_hom_work@yahoo.com>
+    * cpu_info0 doesn't always exist on Solaris, as the first CPU
+    * need not be in slot 0.
+    * Therefore, if ks == NULL after kstat_lookup(), we try
+    * to find the first valid instance using the query:
+    *   ks = kstat_lookup(kc, km_name, -1, NULL);
     */
 
-   if ((km_name == "cpu_info") && (ks == NULL))  {
+   if ((strcmp(km_name, "cpu_info") == 0) && (ks == NULL))  {
       debug_msg( "Lookup up kstat:  km (unix?)='%s', ks (system_misc?)='NULL',kn (resulting metric?)='%s'", km_name, name);
       ks = kstat_lookup(kc, km_name, -1, NULL);
       debug_msg("Just did kstat_lookup() on first instance of module %s.\n",km_name);
@@ -627,7 +630,7 @@ metric_init( void )
 
    get_kstat_val(&metriclist.cpu_num, "unix","system_misc","ncpus");
    debug_msg("metric_init: Assigning cpu_num value (%d) to ncpus.",(int)metriclist.cpu_num.uint32);
-   ncpus = metriclist.cpu_num.uint32;
+   g_ncpus = metriclist.cpu_num.uint32;
 
    get_kstat_val(&metriclist.boottime, "unix","system_misc","boot_time");
    get_kstat_val(&metriclist.cpu_speed,    "cpu_info","cpu_info0","clock_MHz");
