@@ -1,5 +1,5 @@
 <?php
-/* $Id: ganglia.php 809 2007-07-12 06:34:23Z bernardli $ */
+/* $Id: ganglia.php 1817 2008-09-17 10:03:15Z carenas $ */
 #
 # Parses ganglia XML tree.
 #
@@ -148,6 +148,19 @@ function start_cluster ($parser, $tagname, $attrs)
                   $cluster['HOSTS_DOWN']++;
                   $hosts_down[$hostname] = $attrs;
                }
+            # Pseudo metrics - add useful HOST attributes like gmond_started & last_reported to the metrics list:
+            $metrics[$hostname]['gmond_started']['NAME'] = "GMOND_STARTED";
+            $metrics[$hostname]['gmond_started']['VAL'] = $attrs['GMOND_STARTED'];
+            $metrics[$hostname]['gmond_started']['TYPE'] = "timestamp";
+            $metrics[$hostname]['last_reported']['NAME'] = "REPORTED";
+            $metrics[$hostname]['last_reported']['VAL'] = uptime($cluster['LOCALTIME'] - $attrs['REPORTED']);
+            $metrics[$hostname]['last_reported']['TYPE'] = "string";
+            $metrics[$hostname]['ip_address']['NAME'] = "IP";
+            $metrics[$hostname]['ip_address']['VAL'] = $attrs['IP'];
+            $metrics[$hostname]['ip_address']['TYPE'] = "string";
+            $metrics[$hostname]['location']['NAME'] = "LOCATION";
+            $metrics[$hostname]['location']['VAL'] = $attrs['LOCATION'];
+            $metrics[$hostname]['location']['TYPE'] = "string";
             break;
 
          case "METRIC":
@@ -195,6 +208,7 @@ function start_cluster_summary ($parser, $tagname, $attrs)
 function start_host ($parser, $tagname, $attrs)
 {
    global $metrics, $cluster, $hosts_up, $hosts_down, $self, $grid;
+   static $metricname;
 
    switch ($tagname)
       {
@@ -217,7 +231,27 @@ function start_host ($parser, $tagname, $attrs)
             break;
 
          case "METRIC":
-            $metrics[$attrs['NAME']] = $attrs;
+            $metricname = $attrs['NAME'];
+            $metrics[$metricname] = $attrs;
+            break;
+
+         case "EXTRA_DATA":
+            break;
+
+         case "EXTRA_ELEMENT":
+            if ( isset($attrs['NAME']) && isset($attrs['VAL']) && ($attrs['NAME'] == "GROUP")) { 
+               if ( isset($metrics[$metricname]['GROUP']) ) {
+                  $group_array = array_merge( (array)$attrs['VAL'], $metrics[$metricname]['GROUP'] );
+               } else {
+                  $group_array = (array)$attrs['VAL'];
+               }
+               $attribarray = array($attrs['NAME'] => $attrs['VAL']);
+               $metrics[$metricname] = array_merge($metrics[$metricname], $attribarray);
+               $metrics[$metricname]['GROUP'] = $group_array;
+            } else {
+               $attribarray = array($attrs['NAME'] => $attrs['VAL']);
+               $metrics[$metricname] = array_merge($metrics[$metricname], $attribarray);
+            }
             break;
 
          default:
