@@ -1,5 +1,5 @@
 <?php
-/* $Id: header.php 905 2007-12-11 22:38:21Z bernardli $ */
+/* $Id: header.php 1831 2008-09-26 12:18:54Z carenas $ */
 
 # Check if this context is private.
 include_once "auth.php";
@@ -7,9 +7,9 @@ checkcontrol();
 checkprivate();
 
 # RFM - These definitions are here to eliminate "undefined variable"
-# error maessages in ssl_error_log.
+# error messages in ssl_error_log.
 !isset($initgrid) and $initgrid = 0;
-!isset($metric) and $metric = "";
+!isset($metricname) and $metricname = "";
 !isset($context_metrics) and $context_metrics = "";
 
 if ( $context == "control" && $controlroom < 0 )
@@ -17,9 +17,6 @@ if ( $context == "control" && $controlroom < 0 )
 else
       $header = "header";
 
-$tpl = new TemplatePower( template("$header.tpl") );
-$tpl->prepare();
-
 #
 # sacerdoti: beginning of Grid tree state handling
 #
@@ -46,15 +43,18 @@ else if ($gridwalk=="back")
             array_pop($gridstack);
          }
    }
-$gridstack_str = join(":", $gridstack);
+$gridstack_str = join(">", $gridstack);
 $gridstack_url = rawurlencode($gridstack_str);
 
+if (strstr($clustername, "http://")) {
+   header("Location: $clustername?gw=fwd&amp;gs=$gridstack_url");
+}
+
 if ($initgrid or $gridwalk)
    {
       # Use cookie so we dont have to pass gridstack around within this site.
       # Cookie values are automatically urlencoded. Expires in a day.
-      $gscookie = $_COOKIE["gs"];
-      if (! isset($gscookie))
+      if ( !isset($_COOKIE["gs"]) or $_COOKIE["gs"] != $gridstack_str )
             setcookie("gs", $gridstack_str, time() + 86400);
    }
 
@@ -67,21 +67,15 @@ if(count($gridstack) > 1) {
   list($parentgrid, $parentlink) = explode("@", $gridstack[count($gridstack)-2]);
 }
 
-# Setup a redirect to a remote server if you choose a grid from pulldown menu.
-# Tell destination server that we're walking foward in the grid tree.
-if (strstr($clustername, "http://")) 
-   {
-      $tpl->assign("refresh", "0");
-      $tpl->assign("redirect", ";URL=$clustername?gw=fwd&amp;gs=$gridstack_url");
-      echo "<h2>Redirecting, please wait...</h2>";
-      $tpl->printToScreen();
-      exit;
-   }
+$tpl = new TemplatePower( template("$header.tpl") );
+$tpl->prepare();
+$tpl->assign("page_title", $title);
 $tpl->assign("refresh", $default_refresh);
 
-$tpl->assign( "date", date("r"));
-$tpl->assign( "page_title", $title );
+# Templated Logo image
+$tpl->assign("images","./templates/$template_name/images");
 
+$tpl->assign( "date", date("r"));
 
 # The page to go to when "Get Fresh Data" is pressed.
 if (isset($page))
@@ -89,14 +83,11 @@ if (isset($page))
 else
       $tpl->assign("page","./");
 
-# Templated Logo image
-$tpl->assign("images","./templates/$template_name/images");
-
 #
 # Used when making graphs via graph.php. Included in most URLs
 #
 $sort_url=rawurlencode($sort);
-$get_metric_string = "m=$metric&amp;r=$range&amp;s=$sort_url&amp;hc=$hostcols";
+$get_metric_string = "m=$metricname&amp;r=$range&amp;s=$sort_url&amp;hc=$hostcols&amp;mc=$metriccols";
 if ($jobrange and $jobstart)
         $get_metric_string .= "&amp;jr=$jobrange&amp;js=$jobstart";
 
@@ -251,7 +242,7 @@ if (!$physical) {
       ."<SELECT NAME=\"r\" OnChange=\"ganglia_form.submit();\">\n";
    foreach ($context_ranges as $v) {
       $url=rawurlencode($v);
-      $range_menu .= "<OPTION VALUE=\"$url\" ";
+      $range_menu .= "<OPTION VALUE=\"$url\"";
       if ($v == $range)
          $range_menu .= "SELECTED";
       $range_menu .= ">$v\n";
@@ -330,10 +321,39 @@ if ($context == "physical" or $context == "cluster")
             $cols_menu .= ">$cols\n";
          }
       $cols_menu .= "</SELECT>\n";
-
+      
+      $size_menu = '<SELECT NAME="z" OnChange="ganglia_form.submit();">';
+      
+      $size_arr = $graph_sizes_keys;
+      foreach ($size_arr as $size) {
+          if ($size == "default")
+              continue;
+          $size_menu .= "<OPTION VALUE=\"$size\"";
+          if (isset($clustergraphsize) && ($size === $clustergraphsize)) {
+              $size_menu .= " SELECTED";
+          }
+          $size_menu .= ">$size</OPTION>\n";
+      }
+      $size_menu .= "</SELECT>\n";
+  
       # Assign template variable in cluster view.
    }
 
+if ($context == "host")
+   {
+      # Present a width list
+      $metric_cols_menu = "<SELECT NAME=\"mc\" OnChange=\"ganglia_form.submit();\">\n";
+
+      foreach(range(1,25) as $metric_cols)
+         {
+	    $metric_cols_menu .= "<OPTION VALUE=$metric_cols ";
+	    if ($metric_cols == $metriccols)
+	       $metric_cols_menu .= "SELECTED";
+	    $metric_cols_menu .= ">$metric_cols\n";
+	 }
+      $metric_cols_menu .= "</SELECT>\n";
+   }
+
 # Make sure that no data is cached..
 header ("Expires: Mon, 26 Jul 1997 05:00:00 GMT");    # Date in the past
 header ("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT"); # always modified
