@@ -7,7 +7,7 @@
 
 
     /*
-    ** $Id: pie.php 627 2006-03-13 00:19:47Z knobi1 $
+    ** $Id: pie.php 1651 2008-08-10 12:41:37Z carenas $
     **
     ** PHP Class for creating pie charts using the GD library functions
     **
@@ -119,6 +119,8 @@ class piechart
     */
     function init ($w, $h, $d) {
       $this->im= ImageCreate($w, $h);
+      imagesavealpha($this->im, true);
+ 
       $this->width = $w;
       $this->height = $h;
       $this->data = $d;
@@ -135,7 +137,9 @@ class piechart
       } else {
         $this->diameter = $this->da_height;
       }
-      $this->white = ImageColorAllocate($this->im, 255, 255, 255);
+      $this->white = ImageColorAllocateAlpha($this->im, 0, 0, 0, 127);
+      imagefill($this->im, 0, 0, $this->white);
+
       $this->black = ImageColorAllocate($this->im,0,0,0);
       $n = count($this->data);
       for ($i = 0; $i < $n; $i++) {
@@ -201,7 +205,7 @@ class piechart
     function draw_legends ($fontsize) {
       $n = count($this->data);
       $x1 = $this->width - $this->right + $this->legend_left;
-      $x2 = $x1 + $this->fy[$fontsize];;
+      $x2 = $x1 + $this->fy[$fontsize];
       for ($i = 0; $i < $n; $i++) {
       /* determine Y coordinates */
       $y1 = ($i * $this->fy[$fontsize] * 1.5) + $this->top;
@@ -275,9 +279,7 @@ class piechart
 
       for( $j = ($this->center_y+$PIE_THICKNESS); $j > $this->center_y; $j-- ) {
 
-        $from = 0;
-
-        for( $p = 0; $p < $pie_count; $p++ ) {
+        for( $from = 0, $p = 0; $p < $pie_count; $p++, $from = $to ) {
 
           $to = $from + $angles[$p];
 
@@ -295,13 +297,10 @@ class piechart
         
           ImageFilledArc( $this->im, $this->center_x, $j, $this->diameter, $this->diameter, $from, $to, $new_color, IMG_ARC_PIE );
 
-          $from += $angles[$p];
         }
       }
    
-      $from = 0;
-  
-      for( $p = 0; $p < $pie_count; $p++ ) {
+      for( $from = 0, $p = 0; $p < $pie_count; $p++, $from = $to ) {
 
         $to = $from + $angles[$p];
         
@@ -310,10 +309,6 @@ class piechart
         if( $to > 360 )
           $to = 360;
 
-        if ($to > 360) {
-          $to = 360;
-        }
-
         ImageFilledArc( $this->im, $this->center_x, $this->center_y, $this->diameter, $this->diameter, $from, $to, $color, IMG_ARC_PIE );
 
         $from += $angles[$p];
