@@ -1,5 +1,5 @@
 <?php
-# $Id: show_node.php 589 2005-10-10 07:42:26Z knobi1 $
+# $Id: show_node.php 1696 2008-08-16 04:23:56Z carenas $
 #
 # Shows information on a particular node, specified by
 # node host name. Used in concert with the physical_view page.
@@ -32,7 +32,7 @@ $tpl->assign("name", $hostname);
 # $metrics is an array of [Metrics][Hostname][NAME|VAL|TYPE|UNITS|SOURCE].
 
 # Find the host's physical location in the cluster.
-$hostattrs = (is_array($hosts_up)) ? $hosts_up : $hosts_down;
+$hostattrs = ($up) ? $hosts_up : $hosts_down;
 list($rack,$rank,$plane) = findlocation($hostattrs);
 $location = ($rack<0) ? "Unknown" : "Rack $rack, Rank $rank, Plane $plane.";
 $tpl->assign("location",$location);
@@ -67,18 +67,20 @@ $part_max = ($part_max_used) ? "$part_max_used% used." : "Unknown";
 
 # Compute time of last heartbeat from node's dendrite.
 $clustertime=$cluster['LOCALTIME'];
+$tpl->assign("clustertime", strftime("%c", $clustertime));
 $heartbeat=$hostattrs['REPORTED'];
 $age = $clustertime - $heartbeat;
-$s = ($age>1) ? "s" : "";
-$tpl->assign("age","$age second$s");
-if (!$up) {
-   $tpl->assign("down","<br><b>This node is down.</b>");
+if ($age > 3600) {
+   $tpl->assign("age", uptime($age));
+} else {
+   $s = ($age > 1) ? "s" : "";
+   $tpl->assign("age", "$age second$s");
 }
 
 # The these hardware units should be more flexible.
 $s = ($cpu_num>1) ? "s" : "";
 $tpl->assign("s",$s);
-$tpl->assign("cpu", sprintf("%s x %.2f Ghz", $cpu_num, $cpu_speed));
+$tpl->assign("cpu", sprintf("%s x %.2f GHz", $cpu_num, $cpu_speed));
 $tpl->assign("mem", sprintf("%.2f GB", $mem_total_gb));
 $tpl->assign("disk","$disk");
 $tpl->assign("part_max_used", "$part_max");
